package edu.uoc.tfc.j2ee.icampanario.components;

import java.io.Serializable;
import java.lang.reflect.Constructor;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * 
 * Clase fábrica genérica (patrón factory) que obtiene la configuración para
 * los diferentes componentes del fichero components.properties.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class ComponentsFactory implements Serializable {

	/**
	 * serial version uid
	 */
	private static final long serialVersionUID = -4414476636414347107L;

	/**
	 * Log
	 */
	private final static Log log = LogFactory.getLog(ComponentsFactory.class);

	/**
	 * Devuelve una nueva instancia del componente indicado
	 * 
	 * @param component Componente
	 * @return Object Instancia del componente
	 */
	public static Object createInstance(String component) {

		if (component != null && !component.equals("")) {

			log.debug("Create an instance by default of " + component);
			try {
				Configuration configuration = initConfiguration();

				String implDefault = configuration.getString(component + ".impl");
				if (implDefault != null && !implDefault.equals("")) {

					String impl = configuration.getString(component + ".impl." + implDefault);
					if (impl != null && !impl.equals("")) {

						Constructor constructor = Class.forName(impl).getConstructor(new Class[] { Configuration.class });
						configuration = configuration.subset(component + "." + implDefault);
						return constructor.newInstance(new Object[] { configuration });

					} else {
						throw new IllegalArgumentException("Property not found " + component + ".impl." + implDefault);
					}
				} else {
					throw new IllegalArgumentException("Property not found " + component + ".impl");
				}

			} catch (Exception e) {
				throw new RuntimeException(e);
			}

		} else {
			throw new IllegalArgumentException("The argument is null or empty");
		}
	}

	/**
	 * Inicializa el componente a partir de la configuración
	 * 
	 * @return
	 */
	private static CompositeConfiguration initConfiguration() {
		CompositeConfiguration config = new CompositeConfiguration();

		try {
			config.addConfiguration(new PropertiesConfiguration("components.properties"));

		} catch (Exception ex) {
			log.info("File not found components.properties");
		}

		return config;
	}
}
