package edu.uoc.tfc.j2ee.icampanario.components.citizensManager;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.exception.CitizensManagerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;

/**
 * 
 * Implementación que gestiona los habitantes de un municipio
 * simulando la obtención de los mismos del censo. Obtiene los datos
 * de una tabla que se ha añadido al modelo de datos pero al estar
 * la aplicación integrada con un componente (CitizensManager) la
 * obtención de los ciudadanos se podría realziar desde cualquier
 * otra fuente de datos.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class CitizensManagerCenso implements CitizensManager {
	
	/**
	 * Log
	 */
	private final static Log log = LogFactory.getLog(CitizensManagerCenso.class);
	
	/**
	 * Configuración
	 */
	private Configuration config;
	
	/**
	 * Constructor
	 * @param config Configuración
	 */
	public CitizensManagerCenso(Configuration config) {
		super();
		this.config = config;
	}

	/**
	 * Realiza una búsqueda a partir de los datos de un usuario
	 * y devuelve los identificadores de los que coincidan
	 * @param user Datos del usuario a buscar
	 * @return Lista de identificadores de usuarios cuyos datos
	 * coinciden con la búsqueda
	 * @throws CitizensManagerException
	 */
	@Override
	public List<String> find(User user) throws CitizensManagerException {
		
		List<String> userIds = null;
		
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("GRepresentaDataSource");
		EntityManager entityManager = emf.createEntityManager();
		
		// busca los usuarios cuyo identificador (NIF) sea el indicado, normalmente
		// debería devolver sólo uno, pero devuelve una lista por si la implementación
		// necesita devolver todos los que cumplan otra condición, por ejemplo los que
		// residen en una calle, municipio, etc.
		Query query = entityManager.createNamedQuery("findUsersByIdNumber");
    	query.setParameter("idNumber", user.getIdNumber());
    	List<User> resultList = (List<User>) query.getResultList();
    	
    	if (resultList != null) {
    		userIds = new ArrayList<String>();
    		for (User user2 : resultList) {
    			userIds.add(user2.getId());
			}
    	}
    	
    	return userIds;
	}

	/**
	 * Obtiene un usuario cuyo identificador sea el indicado  
	 * @param id Identificador del usuario
	 * @return Usuario
	 * @throws CitizensManagerException
	 */
	@Override
	public User get(String id) throws CitizensManagerException {

		EntityManagerFactory emf = Persistence.createEntityManagerFactory("GRepresentaDataSource");
		EntityManager entityManager = emf.createEntityManager();
		
		Query query = entityManager.createNamedQuery("findUsersById");
    	query.setParameter("id", id);
    	User user = (User) query.getSingleResult();
    	return user;
	}
}
