package edu.uoc.tfc.j2ee.icampanario.components.documentsManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManagerCenso;
import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.exception.DocumentsManagerException;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;

/**
 * 
 * Implementación que persiste los documentos en el sistema
 * de ficheros del sistema operativo. Una vez almacenados los
 * documentos en el modelo de datos propio, la aplicación hará uso
 * de la implementación definida para almacenar los documentos, en
 * este caso en una ruta del disco duro, pero se podría realizar
 * una implementación que los envíe a Alfresco, por ejemplo.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class DocumentsManagerFile implements DocumentsManager {
	
	/**
	 * Log
	 */
	private final static Log log = LogFactory.getLog(CitizensManagerCenso.class);
	
	/**
	 * Configuración
	 */
	private Configuration config;
	
	/**
	 * Constructor
	 * @param config Configuración
	 */
	public DocumentsManagerFile(Configuration config) {
		super();
		this.config = config;
	}

	/**
	 * Persiste el documento
	 * @param document Documento
	 * @throws DocumentsManagerException
	 */
	@Override
	public void save(SignDocument document) throws DocumentsManagerException {
		log.info("DocumentsManagerFile: save document...");
		try {
			String basePath = config.getString(GRepresentaConstants.DOCUMENT_MANAGER_PATH);
			
			//Creamos el directorio
			File dir = new File(basePath + "/" + document.getId());
			dir.mkdirs();
			
			//Creamos el documento
			OutputStream file = new FileOutputStream(basePath + "/" + document.getId() + "/" + document.getName());			
			file.write(document.getContent());
			file.close();
			
		} catch(Exception e) {
			throw new DocumentsManagerException(e);
		}
	}

	/**
	 * Obtiene el documento identificado por el id indicado 
	 * @param id Identificador del documento
	 * @return Documento
	 * @throws DocumentsManagerException
	 */
	@Override
	public SignDocument get(String id) throws DocumentsManagerException {
		log.info("DocumentsManagerFile: get document...");
		SignDocument document = null;
		try {
			String basePath = config.getString(GRepresentaConstants.DOCUMENT_MANAGER_PATH);
			
			File dir = new File(basePath + "/" + id);
			
			InputStream file = new FileInputStream(basePath + "/" + id + "/" + dir.listFiles()[0].getName());
			
			document = new SignDocument();
			document.setContent(GRepresentaUseful.inputStreamToByte(file));
			
			file.close();
			
		} catch (Exception e) {
			throw new DocumentsManagerException(e);
		}
		
		return document;
	}

}
