package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;


/**
 * Clase de dominio que representa un documento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Table(name = "GRP_DOCUMENT")
@Audited
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "Document", propOrder = {
    "id",
    "name",
    "format",
    "signDate",
    "transactionCode",
    "extCode",
    "uri",
    "type"
})
public class Document implements Serializable, Cloneable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = -1532002193099199082L;

	/**
	 * Identificador del documento
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "document_sequence")
	@SequenceGenerator(allocationSize = 1, name = "document_sequence", sequenceName = "GRP_S_DOCUMENT")
	@Column(name = "X_DOCU")
	private Long id;

	/**
	 * Nombre del documento
	 */
	@Column(name = "T_NAME", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
	private String name;

	/**
	 * Formato del documento
	 */
	@Column(name = "T_FORMAT", length = GRepresentaNumericConstants.TEXT_50)
	@XmlElement(required = true)
	private String format;

	/**
	 * Contenido del documento
	 */
	@Column(name = "B_DOCUMENT")
	@Lob
	@XmlTransient
	private byte[] document;

	/**
	 * Fecha de firma del documento
	 */
	@Temporal(TemporalType.DATE)
	@Column(name = "F_SIGN_DATE")
	@XmlElement(required = true)
    @XmlSchemaType(name = "date")
	private Calendar signDate;

	/**
	 * Codigo de transaccion de la firma
	 */
	@Column(name = "C_TRANSACTION", length = GRepresentaNumericConstants.TEXT_100)
	@XmlElement(required = true)
	private String transactionCode;

	/**
	 * PKCS7, recibo de la firma
	 */
	@Column(name = "B_PKCS7")
	@Lob
	@XmlTransient
	private byte[] pkcs7;

	/**
	 * Codigo externo
	 */
	@Column(name = "C_EXT_CODE", length = GRepresentaNumericConstants.TEXT_100)
	@XmlElement(required = true)
	private String extCode;

	/**
	 * Identificador uniforme del documento
	 */
	@Column(name = "T_URI", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
	private String uri;

	/**
	 * Tipo del documento
	 */
	@Column(name = "T_TYPE", length = GRepresentaNumericConstants.TEXT_20)
	@XmlElement(required = true)
	private String type;
	
	/**
	 * Devuelve el id
	 * @return id
	 */
	public Long getId() {
		return id;
	}

	/**
	 * Modifica el id
	 * @param id nuevo id
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * Devuelve el nombre
	 * @return nombre
	 */
	public String getName() {
		return name;
	}

	/**
	 * Modifica el nombre
	 * @param name nuevo nombre
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Devuelve el formato
	 * @return formato
	 */
	public String getFormat() {
		return format;
	}

	/**
	 * Modifica el formato
	 * @param format nuevo formato
	 */
	public void setFormat(String format) {
		this.format = format;
	}

	/**
	 * Devuelve el documento
	 * @return documento
	 */
	public byte[] getDocument() {
		return document;
	}
	
	/**
	 * Modifica el documento
	 * @param document nuevo documento
	 */
	public void setDocument(byte[] document) {
		this.document = document.clone();
	}
	
	/**
	 * Devuelve la fecha de firma
	 * @return fecha de firma
	 */
	public Calendar getSignDate() {
		return signDate;
	}

	/**
	 * Modifica la fecha de firma
	 * @param signDate nueva fecha de firma
	 */
	public void setSignDate(Calendar signDate) {
		this.signDate = signDate;
	}

	/**
	 * Devuelve el codigo de transaccion
	 * @return codigo de transaccion
	 */
	public String getTransactionCode() {
		return transactionCode;
	}

	/**
	 * Modifica el codigo de transaccion
	 * @param transactionCode nuevo codigo
	 */
	public void setTransactionCode(String transactionCode) {
		this.transactionCode = transactionCode;
	}

	/**
	 * Devuelve el pkcs7
	 * @return pkcs7
	 */
	public byte[] getPkcs7() {
		return pkcs7;
	}

	/**
	 * Modifica el pkcs7
	 * @param pkcs7 nuevo pkcs7
	 */
	public void setPkcs7(byte[] pkcs7) {
		this.pkcs7 = pkcs7.clone();
	}

	/**
	 * Devuelve el codigo externo
	 * @return codigo externo
	 */
	public String getExtCode() {
		return extCode;
	}

	/**
	 * Modifica el codigo externo
	 * @param extCode nuevo codigo
	 */
	public void setExtCode(String extCode) {
		this.extCode = extCode;
	}

	/**
	 * Devuelve la uri
	 * @return uri
	 */
	public String getUri() {
		return uri;
	}

	/**
	 * Modifica la uri
	 * @param uri nueva uri
	 */
	public void setUri(String uri) {
		this.uri = uri;
	}

	/**
	 * Devuelve el tipo del documento
	 * @return Tipo del documento
	 */
	public String getType() {
		return type;
	}

	/**
	 * Modifica el tipo del documento
	 * @param type Tipo del documento
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Clona un documento
	 * @return objeto clonado
	 * @throws CloneNotSupportedException si no soporta clonado
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

}
