package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * DataModel para un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class EmpowermentDataModel implements Serializable {

    /**
     * Atributo para serializar
     */
    private static final long serialVersionUID = -8330253913239864925L;

    /**
     * Apoderamiento
     */
    private Empowerment empowerment;

    /**
     * Estado actual
     */
    private State currentState;

    /**
     * Usuario poderdante
     */
    private User grantor;

    /**
     * Usuario apoderado
     */
    private User attorney;

    /**
     * Constructor sin parametros
     */
    public EmpowermentDataModel() { }

    /**
     * Constructor con parametros
     * @param empowerment apoderamiento
     * @param state estado
     * @param grantor poderdante
     * @param attorney apoderado
     */
    public EmpowermentDataModel(Empowerment empowerment, State state, User grantor, User attorney) {

    	this.empowerment = empowerment;
        this.grantor = grantor;
        this.attorney = attorney;
        this.currentState = state;
    }

    /**
     * Devuelve el id
     * @return id
     */
    public Long getId() {
        return empowerment.getId();
    }

    /**
     * Modifica el id
     * @param id nuevo id
     */
    public void setId(Long id) {
        empowerment.setId(id);
    }

    /**
     * Devuelve el id del poderdante
     * @return id del poderdante
     */
    public String getIdGrantor() {
        return empowerment.getIdGrantor();
    }

    /**
     * Modifica el id del poderdante
     * @param idGrantor nuevo id
     */
    public void setIdGrantor(String idGrantor) {
        empowerment.setIdGrantor(idGrantor);
    }

    /**
     * Devuelve el id del apoderado
     * @return id del apoderado
     */
    public String getIdAttorney() {
        return empowerment.getIdAttorney();
    }

    /**
     * Modifica el id del apoderado
     * @param idAttorney nuevo id del apoderado
     */
    public void setIdAttorney(String idAttorney) {
        empowerment.setIdAttorney(idAttorney);
    }

    /**
     * Devuelve fecha inicio vigencia
     * @return fecha de inicio de vigencia
     */
    public Date getValidityInitDate() {
        return empowerment.getValidityInitDate();
    }

    /**
     * Modifica fecha inicio vigencia
     * @param validityInitDate nueva fecha inicio vigencia
     */
    public void setValidityInitDate(Date validityInitDate) {
        empowerment.setValidityInitDate(validityInitDate);
    }

    /**
     * Devuelve fecha fin vigencia
     * @return fecha fin de vigencia
     */
    public Date getValidityEndDate() {
    	return empowerment.getValidityEndDate();
    }

    /**
     * Modifica fecha fin de vigencia
     * @param validityEndDate fecha fin de vigencia
     */
    public void setValidityEndDate(Date validityEndDate) {
    	empowerment.setValidityEndDate(validityEndDate);
    }

    /**
     * Devuelve el procedimiento
     * @return procedimiento
     */
    public Procedure getProcedure() {
        return empowerment.getProcedure();
    }

    /**
     * Modifica el procedimiento
     * @param procedure nuevo procedimiento
     */
    public void setProcedure(Procedure procedure) {
        empowerment.setProcedure(procedure);
    }

    /**
     * Devuelve el tipo de otorgamiento
     * @return tipo de otorgamiento
     */
    public String getGrantingType() {
        return empowerment.getGrantingType();
    }

    /**
     * Modifica el tipo de otorgamiento
     * @param grantingType nuevo tipo de otorgamiento
     */
    public void setGrantingType(String grantingType) {
        empowerment.setGrantingType(grantingType);
    }

    /**
     * Devuelve el tipo de apoderamiento
     * @return tipo de apoderamiento
     */
    public String getEmpowermentType() {
        return empowerment.getEmpowermentType();
    }

    /**
     * Modifica el tipo de apoderamiento
     * @param empowermentType nuevo tipo de apoderamiento
     */
    public void setEmpowermentType(String empowermentType) {
        empowerment.setEmpowermentType(empowermentType);
    }

	/**
	 * Devuelve la descripcion del tipo de otorgamiento
	 * @return descripcion tipo otorgamiento
	 */
	public String getTypeGrantingDesc() {
		return empowerment.getTypeGrantingDesc();
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	public String getValidityEndDateDesc() {
		if (empowerment.getValidityEndDate() != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(empowerment.getValidityEndDate());
		} else {
			return "";
		}
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	public String getValidityInitDateDesc() {
		if (empowerment.getValidityInitDate() != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(empowerment.getValidityInitDate());
		} else {
			return "";
		}
	}

	/**
	 * Modifica el numero de apoderamiento
	 * @param empowermentNumber nuevo numero de apoderamiento
	 */
	public void setEmpowermentNumber(String empowermentNumber) {
		empowerment.setEmpowermentNumber(empowermentNumber);
	}

	/**
	 * Devuelve el numero de apoderamiento
	 * @return numero de apoderamiento
	 */
	public String getEmpowermentNumber() {
		return empowerment.getEmpowermentNumber();
	}

	/**
	 * Modifica el estado actual
	 * @param currentState nuevo estado
	 */
	public void setCurrentState(State currentState) {
		this.currentState = currentState;
	}

	/**
	 * Devuelve el estado actual
	 * @return estado
	 */
	public State getCurrentState() {
		return currentState;
	}

	/**
	 * Modifica el apoderamiento
	 * @param empowerment nuevo apoderamiento
	 */
	public void setEmpowerment(Empowerment empowerment) {
		this.empowerment = empowerment;
	}

	/**
	 * Devuelve el apoderamiento
	 * @return apoderamiento
	 */
	public Empowerment getEmpowerment() {
		return empowerment;
	}

	/**
	 * Modifica el apoderado
	 * @param attorney nuevo apoderado
	 */
	public void setAttorney(User attorney) {
		this.attorney = attorney;
	}

	/**
	 * Devuelve el apoderado
	 * @return apoderado
	 */
	public User getAttorney() {
		return attorney;
	}

	/**
	 * Modifica el poderdante
	 * @param grantor nuevo poderdante
	 */
	public void setGrantor(User grantor) {
		this.grantor = grantor;
	}

	/**
	 * Devuelve el poderdante
	 * @return poderdante
	 */
	public User getGrantor() {
		return grantor;
	}


}
