package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.IdClass;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;


/**
 * Documento asociado a un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Audited
@Table(name = "GRP_EMPOWERMENT_DOCUMENT")
@IdClass(EmpowermentDocumentKey.class)
@NamedQueries({@NamedQuery(name = "findDocumentsForEmpowerment",
		query = "from EmpowermentDocument " +
				"empdoc where empdoc.empowerment.id = :idEmpowerment order by empdoc.document.id"),
			  @NamedQuery(name = "findRelatedEmpowerments",
		query = "select empdoc.empowerment from EmpowermentDocument " +
			  	"empdoc where empdoc.document.id = :idDocument and empdoc.empowerment.id != :idEmpowerment") })
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "EmpowermentDocument", propOrder = {
    "idEmpowerment",
    "idDocument",
    "document"
})
public class EmpowermentDocument implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 4919187693111673827L;

	/**
	 * Identificador de apoderamiento
	 */
	@Id
	private Long idEmpowerment;

	/**
	 * Identificador de estado de apoderamiento
	 */
	@Id
	private Long idDocument;

    /**
     * Apoderamiento
     */
    @ManyToOne
    @JoinColumn(name = "EMPO_X_EMPO", insertable = false, updatable = false)
    @XmlTransient
    private Empowerment empowerment;

	/**
	 * Documento
	 */
	@ManyToOne
	@JoinColumn(name = "DOCU_X_DOCU" , insertable = false, updatable = false)
	@XmlElement(required = true)
	private Document document;


	/**
	 * Devuelve el id de apoderamiento
	 * @return id de apoderamiento
	 */
	public Long getIdEmpowerment() {
		return idEmpowerment;
	}

	/**
	 * Modifica el id de apoderamiento
	 * @param idEmpowerment nuevo id de apoderamiento
	 */
	public void setIdEmpowerment(Long idEmpowerment) {
		this.idEmpowerment = idEmpowerment;
	}

	/**
	 * Devuelve el id del documento
	 * @return id del documento
	 */
	public Long getIdDocument() {
		return idDocument;
	}

	/**
	 * Modifica el id del documento
	 * @param idDocument nuevo id del documento
	 */
	public void setIdDocument(Long idDocument) {
		this.idDocument = idDocument;
	}

	/**
	 * Devuelve el documento asociado al apoderamiento
	 * @return documento asociado al apoderamiento
	 */
	public Document getDocument() {
		return document;
	}

	/**
	 * Modifica el documento asociado al apoderamiento
	 * @param document nuevo documento asociado al apoderamiento
	 */
	public void setDocument(Document document) {
		this.document = document;
	}

	/**
	 * Devuelve el apoderamiento
	 * @return apoderamiento
	 */
	public Empowerment getEmpowerment() {
		return empowerment;
	}

	/**
	 * Modifica el apoderamiento
	 * @param empowerment nuevo apoderamiento
	 */
	public void setEmpowerment(Empowerment empowerment) {
		this.empowerment = empowerment;
	}
}
