package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;

import javax.persistence.Column;

/**
 * Representa la clave primaria de la asociación de 
 * un documento y un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class EmpowermentDocumentKey implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 4424302102018351380L;

	/**
	 * Identificador de apoderamiento
	 */
	@Column(name = "EMPO_X_EMPO")
	private Long idEmpowerment;

	/**
	 * Identificador del documento
	 */
	@Column(name = "DOCU_X_DOCU")
	private Long idDocument;


	/**
	 * Devuelve el id del apoderamiento
	 * @return id del apoderamiento
	 */
	public Long getIdEmpowerment() {
		return idEmpowerment;
	}

	/**
	 * Modifica el id del apoderamiento
	 * @param idEmpowerment nuevo id del apoderamiento
	 */
	public void setIdEmpowerment(Long idEmpowerment) {
		this.idEmpowerment = idEmpowerment;
	}

	/**
	 * Devuelve el id del documento
	 * @return id del documento
	 */
	public Long getIdDocument() {
		return idDocument;
	}

	/**
	 * Modifica el id del documento
	 * @param idDocument nuevo id del documento
	 */
	public void setIdDocument(Long idDocument) {
		this.idDocument = idDocument;
	}

	/**
	 * Hashcode de esta clase
	 * @return hashcode
	 */
	public int hashCode() {
        return (int)((this.getIdEmpowerment() == null
                        ? 0 : this.getIdEmpowerment().hashCode())
                 ^ ((Long) this.getIdDocument()));
    }

	/**
	 * Metodo equals de esta clase
	 * @param otherOb objeto a comparar
	 * @return si es igual o no
	 */
    public boolean equals(Object otherOb) {
        if (this == otherOb) {
            return true;
        }
        if (!(otherOb instanceof EmpowermentDocumentKey)) {
            return false;
        }
        EmpowermentDocumentKey other = (EmpowermentDocumentKey) otherOb;
        return ((this.getIdEmpowerment() == null
                        ? other.idEmpowerment == null : this.getIdEmpowerment().equals
                (other.idEmpowerment)) &&
                ((this.getIdDocument() == null
                		? other.idDocument == null : this.getIdDocument().equals
                (other.idDocument))));
    }

    /**
     * toString
     * @return atributos en string
     */
    public String toString() {
        return idEmpowerment + "-" + idDocument;
    }



}
