package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedNativeQueries;
import javax.persistence.NamedNativeQuery;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;

/**
 * Estado de un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Audited
@Table(name = "GRP_EMPOWERMENT_STATE")
@NamedNativeQueries({@NamedNativeQuery(name = "findStatesForEmpowerment" ,
		query = "select x_emst, empo_x_empo, stat_x_stat, f_init_date, f_end_date from "
		+ "( select 999999999999999999 as x_emst, x_empo as empo_x_empo, 'EXPI' as stat_x_stat, f_end_val as f_init_date, f_end_val as f_end_date "
		+ "from grp_empowerment where f_end_val < :now and x_empo = :idEmpowerment "
		+ "union "
		+ "select x_emst, empo_x_empo, stat_x_stat, f_init_date, f_end_date "
		+ "from grp_empowerment_state join grp_state on x_stat = stat_x_stat "
		+ "where empo_x_empo = :idEmpowerment "
		+ ") states "
		+ "order by f_init_date desc , x_emst desc", resultClass = EmpowermentState.class),
					@NamedNativeQuery(name = "findCurrentStateFotAllEmpowerments",
		query = "select x_emst, empo_x_empo, stat_x_stat, f_init_date, f_end_date "
		+ "from "
		+ "( select 99999999999999999 + x_empo as x_emst, x_empo as empo_x_empo, 'EXPI' as stat_x_stat, f_end_val as f_init_date, " +
				"f_end_val as f_end_date "
		+ "from grp_empowerment where f_end_val < :now "
		+ "union "
		+ "select x_emst, empo_x_empo, stat_x_stat, f_init_date, f_end_date "
		+ "from grp_empowerment_state join grp_state on x_stat = stat_x_stat "
		+ "where empo_x_empo not in (select x_empo as x_empo from grp_empowerment where f_end_val < :now) "
		+ "and f_end_date is null) states order by f_init_date desc, x_emst desc", resultClass = EmpowermentState.class) })
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "EmpowermentState", propOrder = {
    "id",
    "initDate",
    "endDate",
    "initDateDesc",
    "endDateDesc",
    "state"
})
public class EmpowermentState implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 6573886545886647358L;

	/**
	 * Identificador del estado de apoderamiento
	 */
	@Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "empowerment_state_sequence")
    @SequenceGenerator(allocationSize = 1, name = "empowerment_state_sequence", sequenceName = "GRP_S_EMPOWERMENT_STATE")
    @Column(name = "X_EMST")
	private Long id;

	/**
	 * Fecha en la que entro en el estado
	 */
	@Temporal(TemporalType.DATE)
	@Column(name = "F_INIT_DATE")
	@XmlElement(required = true)
    @XmlSchemaType(name = "date")
	private Date initDate;

	/**
	 * Fecha fin del estado
	 */
	@Temporal(TemporalType.DATE)
	@Column(name = "F_END_DATE")
	@XmlElement(required = true)
    @XmlSchemaType(name = "date")
	private Date endDate;

	/**
	 * Apoderamiento
	 */
	@ManyToOne
	@JoinColumn(name = "EMPO_X_EMPO")
	@XmlTransient
	private Empowerment empowerment;

	/**
	 * Estado
	 */
	@ManyToOne
	@JoinColumn(name = "STAT_X_STAT")
	@XmlElement(required = true)
	private State state;

	/**
	 * Devuelve la fecha de inicio del estado
	 * @return fecha de inicio del estado
	 */
	public Date getInitDate() {
		return initDate;
	}

	/**
	 * Modifica la fecha de inicio del estado
	 * @param initDate nueva fecha de inicio del estado
	 */
	public void setInitDate(Date initDate) {
		this.initDate = initDate;
	}

	/**
	 * Devuelve la fecha fin de estado
	 * @return fecha fin de estado
	 */
	public Date getEndDate() {
		return endDate;
	}

	/**
	 * Modifica la fecha fin de estado
	 * @param endDate nueva fecha fin de estado
	 */
	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}

	/**
	 * Devuelve el estado del apoderamiento
	 * @return estado del apoderamiento
	 */
	public State getState() {
		return state;
	}

	/**
	 * Modifica el estado del apoderamiento
	 * @param state nuevo estado del apoderamiento
	 */
	public void setState(State state) {
		this.state = state;
	}

	/**
	 * Devuelve el apoderamiento
	 * @return apoderamiento
	 */
	public Empowerment getEmpowerment() {
		return empowerment;
	}

	/**
	 * Modifica el apoderamiento
	 * @param empowerment nuevo apoderamiento
	 */
	public void setEmpowerment(Empowerment empowerment) {
		this.empowerment = empowerment;
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	@XmlElement(required = true)
	public String getEndDateDesc() {
		if (endDate != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(endDate);
		} else {
			return "";
		}
	}

	/**
	 * Devuelve un string representativo de la fecha
	 * @return descripcion de la fecha
	 */
	@XmlElement(required = true)
	public String getInitDateDesc() {
		if (initDate != null) {
			DateFormat format = new SimpleDateFormat("dd/MM/yyyy");
			return format.format(initDate);
		} else {
			return "";
		}
	}

	/**
	 * Modifica el id de estado de apoderamiento
	 * @param id identificador de estado de apoderamiento
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * Devuelve el identificador de estado de apoderamiento
	 * @return identificador de estado de apoderamiento
	 */
	public Long getId() {
		return id;
	}

}
