package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;

import javax.persistence.Column;

/**
 * Representa la clave primaria de la asociación de un 
 * estado y un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class EmpowermentStateKey implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 8151274004960504709L;

	/**
	 * Identificador de apoderamiento
	 */
	@Column(name = "EMPO_X_EMPO")
	private Long idEmpowerment;

	/**
	 * Identificador de estado de apoderamiento
	 */
	@Column(name = "STAT_X_STAT")
	private String idState;

	/**
	 * Devuelve el id de apoderamiento
	 * @return id de apoderamiento
	 */
	public Long getIdEmpowerment() {
		return idEmpowerment;
	}

	/**
	 * Modifica el id de apoderamiento
	 * @param idEmpowerment nuevo id de apoderamiento
	 */
	public void setIdEmpowerment(Long idEmpowerment) {
		this.idEmpowerment = idEmpowerment;
	}

	/**
	 * Devuelve el id del estado
	 * @return id del estado
	 */
	public String getIdState() {
		return idState;
	}

	/**
	 * Modifica el id del estado
	 * @param idState nuevo id del estado
	 */
	public void setIdState(String idState) {
		this.idState = idState;
	}

	/**
	 * Devuelve el hashcode
	 * @return hashcode
	 */
	public int hashCode() {
        return (int)((this.getIdEmpowerment() == null
                        ? 0 : this.getIdEmpowerment().hashCode())
                 ^ (this.getIdState().hashCode()));
    }

	/**
	 * Metodo equals de la clase
	 * @param otherOb objeto a comparar
	 * @return si es igual o no
	 */
    public boolean equals(Object otherOb) {
        if (this == otherOb) {
            return true;
        }
        if (!(otherOb instanceof EmpowermentStateKey)) {
            return false;
        }
        EmpowermentStateKey other = (EmpowermentStateKey) otherOb;
        return ((this.getIdEmpowerment() == null
                        ? other.idEmpowerment == null : this.getIdEmpowerment().equals
                (other.idEmpowerment)) &&
                ((this.getIdState() == null
                		? other.idState == null : this.getIdState().equals
                (other.idState))));
    }

    /**
     * toString
     * @return to string de la clase
     */
    public String toString() {
        return idEmpowerment + "-" + idState;
    }

}
