package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;

import java.io.Serializable;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedNativeQueries;
import javax.persistence.NamedNativeQuery;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

/**
 * Clase que representa un poder otorgado a un apoderado
 * @author Ismael Campanario Cabrera
 *
 * 	findAllProceduresAvaliables
 *  Procedimientos con fecha fin mayor que la actual y que para el
 *  grantor indicado no hayan superado el numero maximo de apoderamientos
 */
@Entity
@Audited
@Table(name = "GRP_PROCEDURE")
@NamedNativeQueries({@NamedNativeQuery (name = "findAllProceduresAvaliables", query = "select x_proc, c_procedure, l_confirm_emp, l_valid_proc, "
	+ "d_procedure, c_ext_code, n_max_emp, case when count is null then 0 else count end "
    + "from grp_procedure left outer join (select proc_x_proc, count(x_empo) as count from grp_empowerment, grp_empowerment_state where c_grantor = :idGrantor "
    + "and x_empo = empo_x_empo and (f_end_val is null or f_end_val >= :now) and stat_x_stat in ('CONF','PEND') "
    + "and (f_end_date is null or f_end_date >= now()) group by proc_x_proc) "
    + "count_emp on x_proc = proc_x_proc where l_valid_proc = 'S' and (n_max_emp = 0 "
    + "or count is null or n_max_emp > count)", resultClass = Procedure.class),
    				@NamedNativeQuery(name = "procedureAvaliableForGrantor", query = "select x_proc, c_procedure, l_confirm_emp, l_valid_proc, "
	+ "d_procedure, c_ext_code, n_max_emp, case when count is null then 0 else count end "
    + "from grp_procedure left outer join (select proc_x_proc, count(x_empo) as count from grp_empowerment where c_grantor = :idGrantor "
    + "and (f_end_val is null or f_end_val >= :now) group by proc_x_proc) count_emp on x_proc = proc_x_proc "
    + "where x_proc = :idProc and l_valid_proc = 'S' and (n_max_emp = 0 or count is null or n_max_emp > count)", resultClass = Procedure.class) })
@NamedQueries({@NamedQuery(name = "findAllProcedures", query = "from Procedure") })
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "Procedure", propOrder = {
    "code",
    "confirmEmpowerment",
    "confirmEmpowermentDB",
    "deletable",
    "description",
    "extCode",
    "validDB",
    "id",
    "maxEmpowerment"
})
public class Procedure implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 7891399133191011322L;

	/**
	 * Identificador del procedimiento
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "procedure_sequence")
	@SequenceGenerator(allocationSize = 1, name = "procedure_sequence", sequenceName = "GRP_S_PROCEDURE")
	@Column(name = "X_PROC")
	private Long id;

	/**
	 * Codigo de procedimiento
	 */
	@NotNull
	@NotEmpty
	@Column(name = "C_PROCEDURE", length = GRepresentaNumericConstants.TEXT_50)
	@XmlElement(required = true)
	private String code;

	/**
	 * Descripcion del procedimiento
	 */
	@Column(name = "D_PROCEDURE", length = GRepresentaNumericConstants.TEXT_2500)
	@XmlElement(required = true)
	private String description;

	/**
	 * Indica si es necesario confirmar apoderamientos de este procedimiento
	 */
	@Column(name = "L_CONFIRM_EMP", length = GRepresentaNumericConstants.TEXT_1)
	private String confirmEmpowermentDB;

	/**
	 * Para devolver boolean
	 */
	@Transient
	private boolean confirmEmpowerment;

	/**
	 * Procedimiento vigente
	 */
	@Column(name = "L_VALID_PROC", length = GRepresentaNumericConstants.TEXT_1)
	private String validDB;

	/**
	 * Numero maximo de apoderamientos para el procedimiento
	 */
	@Column(name = "N_MAX_EMP")
	private Integer maxEmpowerment;

	/**
	 * Codigo externo
	 */
	@Column(name = "C_EXT_CODE", length = GRepresentaNumericConstants.TEXT_100)
	@XmlElement(required = true)
	private String extCode;

	/**
	 * Se puede borrar
	 */
	@Transient
	private boolean deletable;

	/**
     * Listado de apoderamientos
     */
    @OneToMany(mappedBy = "procedure", fetch = FetchType.EAGER)
    @XmlTransient
    private List<Empowerment> empowerments;

	/**
	 * Devuelve el id del procedimiento
	 * @return id del procedimiento
	 */
	public Long getId() {
		return id;
	}

	/**
	 * Modifica el id del procedimiento
	 * @param id nuevo id del procedimiento
	 */
	public void setId(Long id) {
		this.id = id;
	}

	/**
	 * Devuelve el codigo del procedimiento
	 * @return codigo del procedimiento
	 */
	public String getCode() {
		return code;
	}

	/**
	 * Modifica el codigo del procedimiento
	 * @param code nuevo codigo del procedimiento
	 */
	public void setCode(String code) {
		this.code = code;
	}

	/**
	 * Devuelve la descripcion del procedimiento
	 * @return descripcion del procedimiento
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Modifica la descripcion del procedimiento
	 * @param description nueva descripcion del procedimiento
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Indica si se debe confirmar los apoderamientos
	 * @return si se debe confirmar los apoderamientos
	 */
	public boolean isConfirmEmpowerment() {
		return "S".equalsIgnoreCase(confirmEmpowermentDB);
	}

	/**
	 * Modifica si se debe confirmar los apoderamientos
	 * @param confirmEmpowerment nuevo boleano
	 */
	public void setConfirmEmpowerment(boolean confirmEmpowerment) {
		this.confirmEmpowermentDB = confirmEmpowerment ? "S" : "N";
	}

	/**
	 * Devuelve el numero maximo de apoderamientos
	 * @return numero maximo de apoderamientos
	 */
	public Integer getMaxEmpowerment() {
		return maxEmpowerment;
	}

	/**
	 * Modifica el numero maximo de apoderamientos
	 * @param maxEmpowerment numero maximo numero de apoderamientos
	 */
	public void setMaxEmpowerment(Integer maxEmpowerment) {
		this.maxEmpowerment = maxEmpowerment;
	}

	/**
	 * Devuelve el codigo externo
	 * @return codigo externo
	 */
	public String getExtCode() {
		return extCode;
	}

	/**
	 * Modifica el codigo externo
	 * @param extCode nuevo codigo externo
	 */
	public void setExtCode(String extCode) {
		this.extCode = extCode;
	}

	/**
	 * Modifica la lista de apoderamientos
	 * @param empowerments apoderamientos
	 */
	public void setEmpowerments(List<Empowerment> empowerments) {
		this.empowerments = empowerments;
	}

	/**
	 * Devuelve la lista de apoderamientos
	 * @return lista de apoderamientos
	 */
	public List<Empowerment> getEmpowerments() {
		return empowerments;
	}

	/**
	 * Se podra borrar si no tiene apoderamientos
	 * @return si se puede borrar
	 */
	public boolean isDeletable() {
		if (empowerments != null) {
			return empowerments.isEmpty();
		} else {
			return true;
		}
	}

	/**
	 * Modifica si es vigente o no
	 * @param valid nuevo estado
	 */
	public void setValid(boolean valid) {
		this.validDB = valid ? "S" : "N";
	}

	/**
	 * Si es vigente o no
	 * @return si es vigente o no
	 */
	public boolean isValid() {
		return "S".equalsIgnoreCase(validDB);
	}

	/**
	 * Modifica si se puede borrar
	 * @param deletable nuevo valor
	 */
	public void setDeletable(boolean deletable) {
		this.deletable = deletable;
	}

}
