package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;


/**
 * Representa los distintos estados por los que puede
 * pasar un apoderamiento
 *  
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Audited
@Table(name = "GRP_STATE")
@NamedQuery(name = "findAllState", query = "from State")
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "State", propOrder = {
    "code",
    "name"
})
public class State implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = 5804433495185840493L;

	/**
	 * Codigo identificador del estado
	 */
	@Id
	@Column(name = "X_STAT", length = GRepresentaNumericConstants.TEXT_10)
	@XmlElement(required = true)
	private String code;

	/**
	 * Nombre del estado
	 */
	@Column(name = "T_NAME", length = GRepresentaNumericConstants.TEXT_20)
	@XmlElement(required = true)
	private String name;

	/**
	 * Devuelve el nombre del estado
	 * @return nombre del estado
	 */
	public String getName() {
		return name;
	}

	/**
	 * Modifica el nombre del estado
	 * @param name nuevo nombre del estado
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Modifica el codigo del estado
	 * @param code nuevo codigo de estado
	 */
	public void setCode(String code) {
		this.code = code;
	}

	/**
	 * Devuelve el codigo del estado
	 * @return codigo del estado
	 */
	public String getCode() {
		return code;
	}

	/**
	 * El nombre
	 * @return nombre
	 */
	@Override
	public String toString() {
		return name;
	}

}
