package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedNativeQueries;
import javax.persistence.NamedNativeQuery;
import javax.persistence.NamedQuery;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;

import org.hibernate.envers.Audited;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;

/**
 * Representa un usuario
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Entity
@Audited
@Table(name = "COM_USER")
@NamedNativeQueries({ @NamedNativeQuery (name = "findUsersByIdNumber", 
		query = "select * from COM_USER where C_ID_NUMBER = :idNumber", 
		resultClass = User.class),
		@NamedNativeQuery (name = "findUsersById", 
				query = "select * from COM_USER where X_USER = :id", 
				resultClass = User.class)})
@XmlAccessorType(XmlAccessType.FIELD)
@XmlType(name = "User", propOrder = {
    "id",
    "idNumber",
    "name",
    "surname1",
    "surname2",
    "email",
    "telephone"
})
public class User implements Serializable {
	
	/**
	 * seriual version uid
	 */
	private static final long serialVersionUID = 5974635559128933972L;

	/**
	 * Identificador del usuario
	 */
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "user_sequence")
	@SequenceGenerator(allocationSize = 1, name = "user_sequence", sequenceName = "COM_S_USER")
	@Column(name = "X_USER")
	private String id;

	/**
	 * Número del identificador del usuario (NIF, CIF, etc.)
	 */
	@Column(name = "C_ID_NUMBER", length = GRepresentaNumericConstants.TEXT_10)
	@XmlElement(required = true)
    private String idNumber; 

    /**
     * Nombre del usuario
     */
	@Column(name = "T_NAME", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
    private String name;
    
    /**
     * Primer apellido del usuario
     */
	@Column(name = "T_SURNAME1", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
    private String surname1;

    /**
     * Segundo apellido del usuario
     */
	@Column(name = "T_SURNAME2", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
    private String surname2;

    /**
     * Correo electrónico
     */
	@Column(name = "T_EMAIL", length = GRepresentaNumericConstants.TEXT_255)
	@XmlElement(required = true)
    private String email;

    /**
     * Teléfono
     */
	@Column(name = "T_TELEPHONE", length = GRepresentaNumericConstants.TEXT_20)
	@XmlElement(required = true)
    private String telephone;
    
    /**
     * Certificado digital
     */
	@Transient
    private String certificate;

    
    /**
     * Getters and setters 
     */
    
    public String getCertificate() {
		return certificate;
	}

	public void setCertificate(String certificate) {
		this.certificate = certificate;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getIdNumber() {
		return idNumber;
	}

	public void setIdNumber(String idNumber) {
		this.idNumber = idNumber;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname1() {
		return surname1;
	}

	public void setSurname1(String surname1) {
		this.surname1 = surname1;
	}

	public String getSurname2() {
		return surname2;
	}

	public void setSurname2(String surname2) {
		this.surname2 = surname2;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getTelephone() {
		return telephone;
	}

	public void setTelephone(String telephone) {
		this.telephone = telephone;
	}

	public String getSurname() {
		return surname1 + " " + surname2;
	}
}
