package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services;

import java.util.List;
import java.util.Map;

import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.exception.DocumentsManagerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Document;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentState;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.State;

/**
 * Interfaz del servicio que gestiona los apoderamientos
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public interface EmpowermentQueryServiceDAO {

	/**
	 * Crea un apoderamiento
	 * @param empowerment apoderamiento
	 * @param documentId id del documento
	 * @param empoDocument documento apoderamiento
	 * @return si se crea o no
	 */
	boolean createEmpowerment(Empowerment empowerment, Long documentId, EmpowermentDocument empoDocument);

	/**
     * Modifica el estao anterior y lo pasa a uno nuevo
     * @param empowermentStateOld estado anterior
     * @param empowermentStateNew estado nuevo
     * @return si se ha realizado la operacion de forma correcta
     */
    boolean updateState(EmpowermentState empowermentStateOld, EmpowermentState empowermentStateNew);

    /**
     * Crea un documento para un apoderamiento
     * @param empowerment apoderamiento
     * @param documentId id del documento
     * @param empoDocument documento apoderamiento
     * @return si se ha creado o no
     */
    boolean createEmpowermentDocument(Empowerment empowerment, Long documentId, EmpowermentDocument empoDocument);

    /**
     * Crea un documento
     * @param document documento a crear
     * @throws DocumentsManagerException exception
     */
    void createDocument(Document document) throws DocumentsManagerException;

	/**
	 * Modifica un apoderamiento
	 * @param empowerment apoderamiento
	 * @return si se modifica o no
	 */
	boolean updateEmpowerment(Empowerment empowerment);

	/**
	 * Devuelve la lista de apoderamientos para el poderdante indicado
	 * @param idGrantor id del poderdante
	 * @return lista de apoderamientos
	 */
	List<Empowerment> getEmpowermentsAsGrantor(String idGrantor);

	/**
	 * Devuelve la lista de apoderamientos para el apoderado indicado
	 * @param idAttorney id del apoderado
	 * @return lista de apoderamientos
	 */
	List<Empowerment> getEmpowermentAsAttorney(String idAttorney);

    /**
     * Historico de estados para un apoderamiento
     * @param idEmpowerment id del apoderamiento
     * @return listado de estados
     */
    List<EmpowermentState> getStatesForEmpowerment(Long idEmpowerment);

    /**
     * Documentos de un apoderamiento
     * @param idEmpowerment id del apoderamiento
     * @return listado de documentos
     * @throws DocumentsManagerException exception
     */
    List<EmpowermentDocument> getDocumentsForEmpowerment(Long idEmpowerment) throws DocumentsManagerException;

    /**
     * Apoderamientos relacionados con uno dado
     * @param idEmpowerment id del apoderamiento
     * @param idDocument id del documento
     * @return listado de documentos
     */
    List<Empowerment> getRelatedEmpowerments(Long idEmpowerment, Long idDocument);

    /**
     * Estados de todos los apoderamientos
     * @return listado de estados
     */
    Map<Long,State> getCurrentStateForAllEmpowerment();

    /**
     * Devuelve todos los estados del sistema
     * @return estados
     */
    List<State> getAllStates();

}
