package edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.impl;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Procedure;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.ProcedureQueryServiceDAO;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;

/**
 * Implementación del servicio que gestiona los procedimientos
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("procedureService")
@Scope(ScopeType.SESSION)
public class ProcedureQueryServiceDAOImpl implements ProcedureQueryServiceDAO, Serializable {

	/**
	 * Serial
	 */
	private static final long serialVersionUID = 1427227642828315596L;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * EntityManager
	 */
    @In
    private EntityManager entityManager;

	/**
     * Devuelve todos los procedimientos
     * @param idGrantor poderdante (usuario logado)
     * @return los procedimientos
     */
    public List<Procedure> getAllProcedure(String idGrantor) {

    	if (log.isDebugEnabled()) {
    		log.debug("Calling method getAllProcedure...");
    	}

    	Query query = entityManager.createNamedQuery("findAllProceduresAvaliables");
    	query.setParameter("idGrantor", idGrantor);
    	query.setParameter("now", new Date());
    	return query.getResultList();
    }

    /**
     * Si un procedimiento admite mas apoderamientos
     * @param idGrantor poderdante (usuario logado)
     * @param idProc id procedimiento
     * @return los procedimientos
     */
    public boolean isProcedureAvaliableForGrantor(Long idProc ,String idGrantor) {

    	if (log.isDebugEnabled()) {
    		log.debug("Calling method isProcedureAvaliableForGrantor...");
    	}

    	Query query = entityManager.createNamedQuery("procedureAvaliableForGrantor");
    	query.setParameter("idGrantor", idGrantor);
    	query.setParameter("now", new Date());
    	query.setParameter("idProc", idProc);

    	return !query.getResultList().isEmpty();
    }

    /**
     * Devuelve todos los procedimientos
     * @return los procedimientos
     */
	public List<Procedure> getProcedures() {

		if (log.isDebugEnabled()) {
    		log.debug("Calling method getProcedures...");
    	}

    	return entityManager.createNamedQuery("findAllProcedures").getResultList();
	}

	/**
	 * Crea un procedimiento
	 * @param procedure procedimiento a crear
	 */
	public void create(Procedure procedure) {
		if (procedure.getId() != null) {
			entityManager.merge(procedure);
		} else {
			entityManager.persist(procedure);
		}
	}

	/**
	 * Modifica un procedimiento
	 * @param procedure procedimiento a modificar
	 */
	public void update(Procedure procedure) {
		entityManager.merge(procedure);
	}

	/**
	 * Eliminar un procedimiento
	 * @param procedure procedimiento a eliminar
	 */
	public void delete(Procedure procedure) {
		entityManager.remove(entityManager.find(Procedure.class, procedure.getId()));
	}

	/**
	 * get log
	 * @return log
	 */
	public Log getLog() {
		return log;
	}

	/**
	 * set log
	 * @param log log
	 */
	public void setLog(Log log) {
		this.log = log;
	}

	/**
	 * get entityManager
	 * @return entityManager
	 */
	public EntityManager getEntityManager() {
		return entityManager;
	}

	/**
	 * set entityManager
	 * @param entityManager entityManager
	 */
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

}
