package edu.uoc.tfc.j2ee.icampanario.grepresenta.telematic.actions;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManager;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.exception.CitizensManagerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Document;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentState;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.EmpowermentQueryServiceDAO;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.PDFDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.SignerData;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.SimplePDFDocument;

/**
 * Action para la vista del detalle de un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("detailsAction")
@Scope(ScopeType.SESSION)
public class EmpowermentDetailsAction implements Serializable {


	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = -6863765231503183094L;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * Eventos seam
	 */
	@In
	private Events events;

	/**
	 * Configuracion de la aplicacion
	 */
	@In
	private Properties configurationgr;

	/**
	 * CitizensManager
	 */
	@In(required = false, scope = ScopeType.SESSION)
	private CitizensManager citizensManager;

	/**
	 * Componente mensajes
	 */
	@In(required = false)
	private FacesMessages facesMessages;

	/**
	 * Usuario establecido
	 */
	@In(scope = ScopeType.SESSION, required = false)
	private User user;

	/**
	 * Dao de apoderamientos
	 */
	@In(create = true)
	private EmpowermentQueryServiceDAO empowermentService;

	/**
	 * Apoderamiento actual
	 */
	private Empowerment empowerment;

	/**
	 * Apoderado/poderdante
	 */
	private User auxUser;

	/**
	 * Lista de historico de estados
	 */
	private List<EmpowermentState> states;

	/**
	 * Documentos del apoderamiento
	 */
	private List<EmpowermentDocument> documents;

	/**
	 * Apoderamientos relacionados
	 */
	private List<Empowerment> relatedEmpowerments;

	/**
	 * Fecha auxiliar para modifica fin de vigencia
	 */
	private Date auxEndDate;

	/**
	 * Fecha auxiliar para modifica fin de vigencia
	 */
	private Date auxInitDate;

	/**
	 * Inicializa los detalles del apoderamiento
	 * @param empowerment apoderamiento para ver detalles
	 */
	public void chargeEmpowerment(Empowerment empowerment) {
		this.setEmpowerment(new Empowerment());
		this.setEmpowerment(empowerment);
		setAuxUser(new User());
		try {
			//Si es poderdante sacamos el nombre del apoderado
			if (empowerment.getIdGrantor().equals(this.user.getId())) {
				setAuxUser(citizensManager.get(empowerment.getIdAttorney()));
			} else {
				setAuxUser(citizensManager.get(empowerment.getIdGrantor()));
			}
		} catch (CitizensManagerException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
					"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}

		setStates(new LinkedList<EmpowermentState>());
		setStates(empowermentService.getStatesForEmpowerment(empowerment.getId()));

		setDocuments(new LinkedList<EmpowermentDocument>());
		try {
			setDocuments(empowermentService.getDocumentsForEmpowerment(empowerment.getId()));
		} catch (Exception e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
			"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}

		setRelatedEmpowerments(new LinkedList<Empowerment>());
		setRelatedEmpowerments(empowermentService.getRelatedEmpowerments(empowerment.getId(), documents.get(0).getIdDocument()));
	}

	/**
	 * Descarga el documento
	 * @param doc el documento
	 */
	public void downloadEmpowermentDocument(Document doc) {

		FacesContext context = FacesContext.getCurrentInstance();

		try {

			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/pdf");
			response.setHeader("Content-disposition", "attachment;filename=\"" + doc.getName() + "\"");
            ServletOutputStream out = response.getOutputStream();

            //Si tiene datos de firma
            if (doc.getPkcs7() != null) {

	            SignerData signerData = GRepresentaUseful.getCertificateData(doc.getPkcs7());
	            // Preparación del signerData
	            signerData.setPrefix("");
	            signerData.setTransactionId(doc.getTransactionCode());

	            PDFDocument pdfWithoutSign = new SimplePDFDocument("otorgamiento_firmado.pdf", doc.getDocument());
	            List<SignerData> signers = new LinkedList<SignerData>();
	            signers.add(signerData);

	            String key = GRepresentaConstants.KEY_SIGN;
	            String verifyURL = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_URL);
	            String endpoint = configurationgr.getProperty(GRepresentaConstants.SIGN_ENDPOINT_SERVER);
	            String verifyText = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_TEXT);

	            PDFDocument pdfPie;
				pdfPie = GRepresentaUseful.footerGenerator(pdfWithoutSign, signers, verifyURL, endpoint, key.split(","), verifyText);

				byte[] document = pdfPie.getContent();

				//Si esta pendiente debemos anadir la marca de agua
				if (GRepresentaConstants.STATE_PENDING.equals(states.get(0).getState().getCode())) {

		            String urlReal = ((ServletContext)context.getExternalContext().getContext())
		            	.getRealPath(GRepresentaUseful.getWaterMarkReference(context.getExternalContext().getRequestLocale().getLanguage()));

					GRepresentaUseful.insertWaterMark(document, urlReal, out,
		                    GRepresentaNumericConstants.WATERMARK_POSITION_X, GRepresentaNumericConstants.WATERMARK_POSITION_Y);
		            out.write(pdfPie.getContent());
				}

				out.write(document);

            } else {
            	out.write(doc.getDocument());
            }

            out.flush();
            out.close();
            context.responseComplete();


		} catch (FileNotFoundException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (IOException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (Exception e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
			"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Si se pueden realizar operaciones sobre el apoderamiento
	 * @return si se puede o no
	 */
	public boolean isActionsAvaliable() {
		if (GRepresentaConstants.EMP_TYPE_NOTARIAL.equals(this.empowerment.getEmpowermentType()) ||
				GRepresentaConstants.EMP_TYPE_PRESENTIAL.equals(this.empowerment.getEmpowermentType())) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Es confirmable?
	 * @return si es o no confirmable
	 */
	public boolean isConfirmable() {
		boolean b = false;
		if (states != null && !states.isEmpty() && states.get(0) != null) {
			String state = states.get(0).getState().getCode();
			if (GRepresentaConstants.STATE_PENDING.equals(state) ||
					GRepresentaConstants.STATE_REJECTED.equals(state) ||
					GRepresentaConstants.STATE_WAIVED.equals(state)) {
				b = true;
			}
		}
		return b;
	}

	/**
	 * Es rechazable?
	 * @return si es o no rechazable
	 */
	public boolean isRejectable() {
		boolean b = false;
		if (states != null && !states.isEmpty() && states.get(0) != null) {
			String state = states.get(0).getState().getCode();
			if (GRepresentaConstants.STATE_PENDING.equals(state)) {
				b = true;
			}
		}
		return b;
	}

	/**
	 * Es renunciable?
	 * @return si es o no renunciable
	 */
	public boolean isWaived() {
		boolean b = false;
		if (states != null && !states.isEmpty() && states.get(0) != null) {
			String state = states.get(0).getState().getCode();
			if (GRepresentaConstants.STATE_CONFIRM.equals(state)) {
				b = true;
			}
		}
		return b;
	}

	/**
	 * Esta revocado?
	 * @return si esta o no revocado
	 */
	public boolean isRevoked() {
		boolean b = false;
		if (states != null && !states.isEmpty() && states.get(0) != null) {
			String state = states.get(0).getState().getCode();
			if (GRepresentaConstants.STATE_REVOKED.equals(state)) {
				b = true;
			}
		}
		return b;
	}

	/**
	 * Esta expirado?
	 * @return si esta o no expirado
	 */
	public boolean isExpired() {
		boolean b = false;
		if (states != null && !states.isEmpty() && states.get(0) != null) {
			String state = states.get(0).getState().getCode();
			if (GRepresentaConstants.STATE_EXPIRED.equals(state)) {
				b = true;
			}
		}
		return b;
	}

	/**
	 * Modifica el estado del apoderamiento
	 * @param newStateCode nuevo estado
	 * @return siguiente pantalla
	 */
	public String updateEmpowermentState(String newStateCode) {
		String next = null;
		if (newStateCode.equals(GRepresentaConstants.STATE_CONFIRM)) {

			events.raiseEvent("prepareConfirmEmpowerment", states.get(0), auxUser);
			next = "needConfirmation";

		} else if (newStateCode.equals(GRepresentaConstants.STATE_WAIVED)) {

			events.raiseEvent("prepareWaiveEmpowerment", states.get(0), auxUser);
			next = "waive";

		} else if (newStateCode.equals(GRepresentaConstants.STATE_REJECTED)) {

			events.raiseEvent("prepareRejectEmpowerment", states.get(0), auxUser);
			next = "reject";

		} else {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
			"#{messages['message.error.administrator']}");
			log.error("Illegal change of state");
		}
		return next;
	}

	/**
	 * Modifica el apoderamiento de detalles
	 * @param empowerment nuevo apoderamiento
	 */
	public void setEmpowerment(Empowerment empowerment) {
		this.empowerment = empowerment;
	}

	/**
	 * Devuelve el apoderamiento de detalles
	 * @return apoderamiento
	 */
	public Empowerment getEmpowerment() {
		return empowerment;
	}

	/**
	 * Modifica la lista de estados del apoderamiento
	 * @param states nuevos estados de apoderamientos
	 */
	public void setStates(List<EmpowermentState> states) {
		this.states = states;
	}

	/**
	 * Devuelve los estados de una apoderamiento
	 * @return estados de apoderamiento
	 */
	public List<EmpowermentState> getStates() {
		return states;
	}

	/**
	 * Modifica los documentos de un apoderamiento
	 * @param documents nuevos documentos
	 */
	public void setDocuments(List<EmpowermentDocument> documents) {
		this.documents = documents;
	}

	/**
	 * Devuelve los documentos de un apoderamiento
	 * @return los documentos
	 */
	public List<EmpowermentDocument> getDocuments() {
		return documents;
	}

	/**
	 * Modifica la fecha fin de vigencia
	 * @param auxEndDate nueva fecha fin de vigencia
	 */
	public void setAuxEndDate(Date auxEndDate) {
		this.auxEndDate = auxEndDate;
	}

	/**
	 * Devuelve la fecha fin de vigencia
	 * @return fecha fin de vigencia
	 */
	public Date getAuxEndDate() {
		return auxEndDate;
	}

	/**
	 * Modifica el usuario poderdante o apoderado
	 * @param auxUser nuevo usuario poderdante o apoderado
	 */
	public void setAuxUser(User auxUser) {
		this.auxUser = auxUser;
	}

	/**
	 * Devuelve el usuario poderdante o apoderado
	 * @return usuario poderdante o apoderado
	 */
	public User getAuxUser() {
		return auxUser;
	}

	/**
	 * Modifica la lista de apoderamientos relacionados
	 * @param relatedEmpowerments nueva lista
	 */
	public void setRelatedEmpowerments(List<Empowerment> relatedEmpowerments) {
		this.relatedEmpowerments = relatedEmpowerments;
	}

	/**
	 * Devuelve la lista de apoderamientos relacionados
	 * @return apoderamientos relacionados
	 */
	public List<Empowerment> getRelatedEmpowerments() {
		return relatedEmpowerments;
	}

	/**
	 * Modifica la fecha inicio de vigencia
	 * @param auxInitDate nueva fecha inicio de vigencia
	 */
	public void setAuxInitDate(Date auxInitDate) {
		this.auxInitDate = auxInitDate;
	}

	/**
	 * Devuelve la fecha inicio de vigencia
	 * @return fecha inicio de vigencia
	 */
	public Date getAuxInitDate() {
		return auxInitDate;
	}
}
