package edu.uoc.tfc.j2ee.icampanario.grepresenta.telematic.actions;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.validator.ValidatorException;
import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringUtils;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManager;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.exception.CitizensManagerException;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignTransaction;
import edu.uoc.tfc.j2ee.icampanario.components.signer.Signer;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignerFactory;
import edu.uoc.tfc.j2ee.icampanario.components.signer.exception.SignerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Document;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Procedure;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.EmpowermentQueryServiceDAO;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.ProcedureQueryServiceDAO;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.PDFDocument;

/**
 * Action para crear un nuevo apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("newEmpAction")
@Scope(ScopeType.SESSION)
public class NewEmpowermentAction implements Serializable {

    /**
     * Atributo para serializar
     */
    private static final long serialVersionUID = -4345504483272429383L;

    /**
     * Configuracion de la aplicacion
     */
    @In
    private Properties configurationgr;

    /**
     * CitizensManager
     */
    @In(required = false, scope = ScopeType.SESSION)
    private CitizensManager citizensManager;

    /**
     * Log
     */
    @Logger
    private Log log;

    /**
     * Componente mensajes
     */
    @In(required = false)
    private FacesMessages facesMessages;

    /**
     * Mensajes para internacionalización de la aplicación.
     */
    @In
    private Map<String, String> messages;

    /**
     * Eventos seam
     */
    @In
    private Events events;

    /**
     * Usuario establecido
     */
    @In(scope = ScopeType.SESSION, value = "user", required = false)
    private User grantor;

    /**
     * Dao de procedimientos
     */
    @In(create = true)
    private ProcedureQueryServiceDAO procedureService;

    /**
     * Dao de apoderamientos
     */
    @In(create = true)
    private EmpowermentQueryServiceDAO empowermentService;

    /**
     * Lista de apoderamientos posibles a crear
     */
    private List<Empowerment> empowerments = new LinkedList<Empowerment>();

    /**
     * Apoderado
     */
    private User attorney = new User();

    /**
     * Lista de apoderados
     */
    private List<User> attorneyList = new ArrayList<User>();

    /**
     * Si se ha encontrado o no el apoderado
     */
    private boolean attorneyFound;

    /**
     * Nombre del apoderado.
     */
    private String attorneyName;

    /**
     * Si el atributo attorneyFound es false y hay mensaje
     */
    private boolean showMessageNotFound;

    /**
     * Para la seleccion de todos los procedimientos
     */
    private boolean allSelected;

    /**
     * Transaction
     */
    private SignTransaction transaction;

    /**
     * Atributo para firma
     */
    private String clientSignerConfig ;

    /**
     * Documento
     */
    private Document grantingDoc;

    /**
     * Carga todos los procedimientos disponibles
     */
    public void chargeProcedures() {

        log.info("Getting procedures...");

        List<Procedure> procedures = procedureService.getAllProcedure(grantor.getId());
        empowerments = new LinkedList<Empowerment>();
        //Siempre que entremos se vacia el mensaje
        attorneyFound = false;
        showMessageNotFound = false;
        for (Procedure procedure : procedures) {
            empowerments.add(new Empowerment(procedure, false));
        }
        attorney = new User();
        attorneyName = "";

    }

    /**
     * Validador de Nif
     * @param context Context
     * @param component Component
     * @param value Value
     */
    public void validatorNif(FacesContext context, UIComponent component, Object value) {

        log.info("validatorNif...");

        attorneyList = new ArrayList<User>();

        // Si el valor es null, lo transformamos en un valor vacío
        String valor = StringUtils.defaultString((String)value);

        // el valor debe tener 9 posiciones, de las cuales las primeras deben ser dígitos y la última letra
        valor = valor.toUpperCase();

        Pattern mask =  Pattern.compile("[0-9]{8,8}[A-Z]");
        Matcher matcher = mask.matcher(valor);

        if (!matcher.matches()) {
            attorney = new User();
            attorneyFound = false;
            showMessageNotFound = true;
            throw new ValidatorException(new FacesMessage(messages.get("message.error.nif")));
        }

        String dni = valor.substring(0, GRepresentaNumericConstants.TEXT_8);
        String digitoControl = valor.substring(GRepresentaNumericConstants.TEXT_8, GRepresentaNumericConstants.TEXT_9);

        // Calculamos la letra de control
        String letras = "TRWAGMYFPDXBNJZSQVHLCKE";

        int posicionModulo = Integer.parseInt(dni) % GRepresentaNumericConstants.TEXT_23;
        String digitoControlCalculado = letras.substring(posicionModulo,posicionModulo + 1);

        if (!digitoControl.equalsIgnoreCase(digitoControlCalculado)) {
            attorney = new User();
            attorneyFound = false;
            showMessageNotFound = true;
            throw new ValidatorException(new FacesMessage(messages.get("message.error.nif")));
        }
    }

    /**
     * Busca en el sistema externo el apoderado
     * @param event evento
     */
    public void lookForAttorney(ActionEvent event) {

        log.info("Looking for the attorney...");

        //Vaciamos el apoderado si vamos a buscar uno nuevo
        String strValueIdNumber = attorney.getIdNumber();
        attorney = new User();
        attorneyList = new ArrayList<User>();
        if (strValueIdNumber != null && !"".equals(strValueIdNumber)
                && attorneyName != null && !"".equals(attorneyName)) {

            attorney.setIdNumber(strValueIdNumber);

            List<String> ids = new LinkedList<String>();

            try {
                ids = citizensManager.find(attorney);

                if (ids.isEmpty()) {
                    attorneyFound = false;
                    showMessageNotFound = true;
                } else {
                    for (String id : ids) {
                            attorneyList.add(citizensManager.get(id));
                    }

                    User user = new User();

                    int percentage = GRepresentaUseful.getSimilarUser(attorneyList, user, attorneyName);

                    if (percentage >= GRepresentaNumericConstants.TEXT_80 && user != null) {
                        attorney.setId(user.getId());
                        attorneyFound = true;
                    } else {
                        attorneyFound = false;
                        showMessageNotFound = true;
                    }
                }
            } catch (CitizensManagerException e) {
                facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
                log.error(e.getMessage(), e);
                attorneyFound = false;
                showMessageNotFound = true;
            }
        } else {
            attorneyFound = false;
            showMessageNotFound = false;
        }
    }

    /**
     * Selecciona todos los procedimientos
     * @param event evento
     */
    public void selectAllProcedures(ActionEvent event) {
        for (Empowerment emp : empowerments) {
            emp.setSelected(allSelected);
        }
    }

    /**
     * Comprobaciones antes de pasar a pantalla de confirmacion
     * @return siguiente pantalla
     */
    public String validateEmpowerment() {
        String success = "confirmEmpowerment";

        //Validamos que exista un apoderado
        if (!attorneyFound) {
            facesMessages.add(Severity.ERROR, "#{messages['message.thereIsNotAttorneySelected']}",
                    "#{messages['message.thereIsNotAttorneySelected']}");
            success = null;
        }

        //Validamos que el apoderado no sea el poderdante
        if (attorney.getIdNumber().equals(grantor.getIdNumber()) && success != null) {
            facesMessages.add(Severity.ERROR, "#{messages['message.idAttorneySameIdGrantor']}",
                    "#{messages['message.idAttorneySameIdGrantor']}");
            success = null;
        }

        //Validamos que haya al menos un procedimiento seleccionado y que si un procedimiento es confirmable
        //o tipo tramites, todos son confirmables o tipos tramites
        if (success != null) {
            boolean oneSelected = false;
            boolean oneConfirmable = false;
            boolean oneNotConfirmable = false;
            boolean dateRangeError = false;
            boolean dateInitNull = false;

            for (Empowerment emp : empowerments) {
                if (emp.isSelected()) {
                    oneSelected = true;
                    if (emp.getProcedure().isConfirmEmpowerment() ||
                            !GRepresentaConstants.GRANTING_PROCEDURE.equals(emp.getGrantingType())) {
                        oneConfirmable = true;
                    } else {
                        oneNotConfirmable = true;
                    }

                    if (emp.getValidityInitDate() == null) {
                        dateInitNull = true;
                    }

                    if (emp.getValidityEndDate() != null &&
                        emp.getValidityInitDate() != null &&
                            emp.getValidityEndDate().before(emp.getValidityInitDate())) {
                                dateRangeError = true;
                    }
                }
            }

            if (!oneSelected) {
                facesMessages.add(Severity.ERROR, "#{messages['message.thereIsNotProcedureSelected']}",
                        "#{messages['message.thereIsNotProcedureSelected']}");
                success = null;
            }

            if (oneConfirmable && oneNotConfirmable) {
                facesMessages.add(Severity.ERROR, "#{messages['message.allConfirmableNeeded']}",
                    "#{messages['message.allConfirmableNeeded']}");
                success = null;
            }

            if (dateRangeError) {
                facesMessages.add(Severity.ERROR, "#{messages['message.dateRangeError']}",
                    "#{messages['message.dateRangeError']}");
                success = null;
            }

            if (dateInitNull) {
                facesMessages.add(Severity.ERROR, "#{messages['message.dateInitNull']}",
                "#{messages['message.dateInitNull']}");
                success = null;
            }
        }

        //Buscamos el apoderado
        if (success != null) {
            try {
                attorney = citizensManager.get(attorney.getId());
            } catch (CitizensManagerException e) {
                facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                    "#{messages['message.error.administrator']}");
                log.error(e.getMessage(), e);
                success = null;
            }
        }

        if (success != null) {
            prepareDocumentForSign();
        }

        return success;
    }

    /**
     * Prepara el documento para la firma
     */
    public void prepareDocumentForSign() {

        FacesContext context = FacesContext.getCurrentInstance();
        String idiom = context.getExternalContext().getRequestLocale().getLanguage();

        String host = configurationgr.getProperty(GRepresentaConstants.OPENOFFICE_URL);
        int port = Integer.parseInt(configurationgr.getProperty(GRepresentaConstants.OPENOFFICE_PORT));

        InputStream openDocument = null;

        try {

            String separator = System.getProperty("file.separator");
            StringWriter urlRelative = new StringWriter();
            urlRelative.append("WEB-INF");
            urlRelative.append(separator);
            urlRelative.append("classes");
            urlRelative.append(separator);
            urlRelative.append("public");
            urlRelative.append(separator);
            urlRelative.append("forms_");
            urlRelative.append(idiom);
            urlRelative.append(separator);
            urlRelative.append("otorgamiento_telematica.odt");

            String urlReal = ((ServletContext)context.getExternalContext().getContext()).getRealPath(urlRelative.toString());

            if (urlReal != null) {
                openDocument = new FileInputStream(urlReal);

                Map<String,Object> vars = new HashMap<String,Object>();

                // variables comunes
                vars.put("poderdante", grantor.getName() + " " + grantor.getSurname());
                vars.put("nif_poderdante", grantor.getIdNumber());
                vars.put("apoderado", attorney.getName() + " " + attorney.getSurname());
                vars.put("nif_apoderado", attorney.getIdNumber());

                // procedimientos otorgamiento
                List<Empowerment> procSelected = new LinkedList<Empowerment>();
                for (Empowerment emp : empowerments) {
                    if (emp.isSelected()) {
                        procSelected.add(emp);
                    }
                }
                vars.put("tabla_procedimientos", procSelected);

                HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();

                byte [] tmpFile = GRepresentaUseful.setVariables(host, port, openDocument, vars);
                ByteArrayOutputStream out = new ByteArrayOutputStream(tmpFile.length);
                GRepresentaUseful.generatePDF(host, port, out, tmpFile);

                //Preparacion para la firma
                Signer signer = SignerFactory.createInstance();

                transaction = new SignTransaction();
                SignDocument doc = new SignDocument();
                doc.setContent(out.toByteArray());
                doc.setMimeType("application/pdf");
                doc.setName("otorgamiento_telematico.pdf");
                transaction.addDocument(doc);
                transaction.setUser(grantor.getCertificate());
                signer.initSign(transaction, request);

                clientSignerConfig = (String)request.getAttribute("clientSignerConfig");

            } else {
                facesMessages.add(Severity.ERROR, "#{messages['message.notFileFound']} " + urlRelative,
                        "#{messages['message.notFileFound']} " + urlRelative);
            }


        } catch (FileNotFoundException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (Exception e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        }
    }

    /**
     * Genera el pdf con los datos de los apoderamientos seleccionados
     */
    public void generateNewEmpowermentPdf() {

        FacesContext context = FacesContext.getCurrentInstance();

        try {

            HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
            response.setContentType("application/pdf");
            response.setHeader("Content-disposition", "attachment;filename=\"otorgamiento_telematico.pdf\"");

            ServletOutputStream out = response.getOutputStream();

            out.write(transaction.getDocuments().get(0).getContent());

            out.flush();
            context.responseComplete();


        } catch (FileNotFoundException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        }

    }

    /**
     * Realiza la firma digital del documento
     * @return siguiente pantalla
     */
    public String signDocument() {

        String next = null;

        HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();

        Signer signer = SignerFactory.createInstance();

        boolean pend = false;

        try {

            for (Empowerment emp : empowerments) {
                    //Si el procedimiento necesita confirmacion o el tipo de apoderamiento es tramites(necesita confirmacion)
                if (emp.isSelected() && (emp.getProcedure().isConfirmEmpowerment()
                        || !emp.getGrantingType().equals(GRepresentaConstants.GRANTING_PROCEDURE))) {
                        pend = true;
                    }
                }

            signer.sign(transaction, request);

            if (pend) {
            	createEmpowerments();
            	next = "showSignedDocumentWithWaterMark";
            } else {
            	if (signAndRegisterDocument()) {
            		next = "showSignedDocumentEmpowerment";
            	}
            }

        } catch (SignerException e) {
            log.error(e.getMessage(), e);
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
        }
        return next;
    }

    /**
     * Realiza la firma digital del documento y lo almacena
     * @return boolean
     */
    public boolean signAndRegisterDocument() {

        grantingDoc = null;
        boolean signAndReg = false;

        try {
            createEmpowerments();

            // procedimientos otorgamiento
            StringBuffer numExp = new StringBuffer();
            StringBuffer proc = new StringBuffer();
            for (Empowerment emp : empowerments) {
                if (emp.isSelected()) {
                    if (proc != null && proc.length() == 0) {
                        proc.append(emp.getProcedure().getDescription());
                    } else {
                        proc.append(", ");
                        proc.append(emp.getProcedure().getDescription());
                    }
                    if (numExp != null && numExp.length() == 0) {
                        numExp.append(String.valueOf(emp.getId()));
                    } else {
                        numExp.append(", ");
                        numExp.append(String.valueOf(emp.getId()));
                    }
                }
            }

            SignDocument signDocument = new SignDocument();
            signDocument.setContent(grantingDoc.getDocument());
            signDocument.setName("otorgamiento");
            signDocument.setMimeType("application/pdf");

            empowerments = new LinkedList<Empowerment>();

            signAndReg = true;

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
        }

        return signAndReg;

    }

    /***
     * Crea los apoderamientos
     */
    public void createEmpowerments() {

        grantingDoc = new Document();
        grantingDoc.setDocument(transaction.getDocuments().get(0).getContent());
        grantingDoc.setFormat(transaction.getDocuments().get(0).getMimeType());
        grantingDoc.setPkcs7(Base64.decodeBase64(transaction.getDocuments().get(0).getSign()));
        grantingDoc.setSignDate(new GregorianCalendar());
        grantingDoc.setTransactionCode(transaction.getDocuments().get(0).getId());
        grantingDoc.setName("otorgamiento");
        grantingDoc.setType(GRepresentaConstants.DOC_TYPE_REQUEST);

        //Se crea el documento para luego relacionarlo
        try {
			empowermentService.createDocument(grantingDoc);
		} catch (Exception e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
            "#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}

        //Para que se creen todos a la vez
        Date now = new Date();
        boolean create = true;

        for (Empowerment emp : empowerments) {
            if (emp.isSelected()) {
                emp.setIdGrantor(grantor.getId());
                emp.setIdAttorney(attorney.getId());
                if (emp.getValidityInitDate() == null) {
                    emp.setValidityInitDate(now);
                }
                emp.setEmpowermentType(GRepresentaConstants.EMP_TYPE_TELEMATIC);
                if (!empowermentService.createEmpowerment(emp, grantingDoc.getId(), null)) {
                    create = false;
                }
            }
        }

        if (!create) {
            facesMessages.add(Severity.ERROR,"#{messages['message.errorCreateEmpowerment']}");
        }

        //Se limpian los atributos
        attorney = new User();
        allSelected = false;

        events.raiseEvent("refreshGrantorList");

    }

    /**
     * Genera el pdf firmado para mostrarlo
     */
    public void generateNewEmpowermentSignedPdf() {

        FacesContext context = FacesContext.getCurrentInstance();

        try {

            HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
            response.setContentType("application/pdf");
            response.setHeader("Content-disposition", "attachment;filename=\"otorgamiento_telematico_firmado.pdf\"");

            ServletOutputStream out = response.getOutputStream();

            String key = GRepresentaConstants.KEY_SIGN;
            String verifyURL = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_URL);
            String endpoint = configurationgr.getProperty(GRepresentaConstants.SIGN_ENDPOINT_SERVER);
            String verifyText = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_TEXT);

            PDFDocument pdfPie = GRepresentaUseful.generatePdtWithFoot(transaction.getDocuments().get(0), key, verifyURL, endpoint, verifyText);

            out.write(pdfPie.getContent());

            out.flush();
            out.close();
            context.responseComplete();

        } catch (FileNotFoundException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
                "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (Exception e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
            "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        }
    }

    /**
     * Genera el pdf firmado para mostrarlo con marca de agua
     */
    public void generateNewEmpowermentSignedPdfWaterMark() {

        FacesContext context = FacesContext.getCurrentInstance();

        try {

            HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
            response.setContentType("application/pdf");
            response.setHeader("Content-disposition", "attachment;filename=\"otorgamiento_telematico_firmado.pdf\"");

            String idiom = context.getExternalContext().getRequestLocale().getLanguage();

            ServletOutputStream out = response.getOutputStream();

            String key = GRepresentaConstants.KEY_SIGN;
            String verifyURL = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_URL);
            String endpoint = configurationgr.getProperty(GRepresentaConstants.SIGN_ENDPOINT_SERVER);
            String verifyText = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_TEXT);

            PDFDocument pdfPie = GRepresentaUseful.generatePdtWithFoot(transaction.getDocuments().get(0), key, verifyURL, endpoint, verifyText);

            String separator = System.getProperty("file.separator");
            StringWriter urlRelative = new StringWriter();
            urlRelative.append("WEB-INF");
            urlRelative.append(separator);
            urlRelative.append("classes");
            urlRelative.append(separator);
            urlRelative.append("public");
            urlRelative.append(separator);
            urlRelative.append("forms_");
            urlRelative.append(idiom);
            urlRelative.append(separator);
            urlRelative.append("watermark.gif");

            String urlReal = ((ServletContext)context.getExternalContext().getContext()).getRealPath(urlRelative.toString());

            GRepresentaUseful.insertWaterMark(pdfPie.getContent(), urlReal, out,
                    GRepresentaNumericConstants.WATERMARK_POSITION_X, GRepresentaNumericConstants.WATERMARK_POSITION_Y);
            out.write(pdfPie.getContent());

            out.flush();
            out.close();
            context.responseComplete();

        } catch (FileNotFoundException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
            "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (IOException e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
            "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        } catch (Exception e) {
            facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
            "#{messages['message.error.administrator']}");
            log.error(e.getMessage(), e);
        }    
    }

    /**
     * Limpia los datos de sesión.
     */
    public void clearData() {
        //Se limpian los atributos
        transaction = new SignTransaction();
        clientSignerConfig = null;
        grantingDoc = null;
        attorneyName = "";
    }

    /**
     * Modifica el apoderado
     * @param attorney nuevo apoderado
     */
    public void setAttorney(User attorney) {
        this.attorney = attorney;
    }

    /**
     * Devuelve el apoderado
     * @return apoderado
     */
    public User getAttorney() {
        return attorney;
    }

    /**
     * Modifica el poderdante
     * @param grantor nuevo poderdante
     */
    public void setGrantor(User grantor) {
        this.grantor = grantor;
    }

    /**
     * Devuelve el poderdante
     * @return poderdante
     */
    public User getGrantor() {
        return grantor;
    }

    /**
     * @param facesMessages mensajes
     */
    public void setFacesMessages(FacesMessages facesMessages) {
        this.facesMessages = facesMessages;
    }

    /**
     * @return mensajes
     */
    public FacesMessages getFacesMessages() {
        return facesMessages;
    }

    /**
     * Devuelve la lista de apoderamientos que se esta creando
     * @return apoderamientos
     */
    public List<Empowerment> getEmpowerments() {
        return empowerments;
    }

    /**
     * Modifica los apoderamientos
     * @param empowerments nuevos apoderamientos
     */
    public void setEmpowerments(List<Empowerment> empowerments) {
        this.empowerments = empowerments;
    }

    /**
     * Indica si esta todo seleccionado
     * @return si esta o no
     */
    public boolean isAllSelected() {
        return allSelected;
    }

    /**
     * Modifica el seleccionado
     * @param allSelected nuevo valor
     */
    public void setAllSelected(boolean allSelected) {
        this.allSelected = allSelected;
    }

    /**
     * Modifica si se ha encontrado el apoderado
     * @param attorneyFound nuevo parametro
     */
    public void setAttorneyFound(boolean attorneyFound) {
        this.attorneyFound = attorneyFound;
    }

    /**
     * Devuelve si se ha encontrado el apoderado
     * @return si se ha encontrado
     */
    public boolean isAttorneyFound() {
        return attorneyFound;
    }

    /**
     * Modifica el mostrar mensaje de no encontrado
     * @param showMessageNotFound nuevo valor
     */
    public void setShowMessageNotFound(boolean showMessageNotFound) {
        this.showMessageNotFound = showMessageNotFound;
    }

    /**
     * Si esta el mensaje de no encontrado
     * @return si esta o no
     */
    public boolean isShowMessageNotFound() {
        return showMessageNotFound;
    }

    /**
     * Modifica la configuracion de la aplicacion
     * @param configurationgr nueva configuracion
     */
    public void setConfigurationgr(Properties configurationgr) {
        this.configurationgr = configurationgr;
    }

    /**
     * Devuelve la configuracion de la aplicacion
     * @return configuracion de la aplicacion
     */
    public Properties getConfigurationgr() {
        return configurationgr;
    }

    /**
     * Devuelve la transaccion
     * @return la transaccion
     */
    public SignTransaction getTransaction() {
        return transaction;
    }

    /**
     * Modifica la transaccion
     * @param transaction nueva transaccion
     */
    public void setTransaction(SignTransaction transaction) {
        this.transaction = transaction;
    }

    /**
     * Modifica el atributo para la firma
     * @param clientSignerConfig nuevo atributo para la firma
     */
    public void setClientSignerConfig(String clientSignerConfig) {
        this.clientSignerConfig = clientSignerConfig;
    }

    /**
     * Devuelve el atributo para la firma
     * @return atributo para la firma
     */
    public String getClientSignerConfig() {
        return clientSignerConfig;
    }

    /**
     * Devuelve la lista con los apoderados.
     * @return	Lista con los apoderados.
     */
    public List<User> getAttorneyList() {
        return attorneyList;
    }

    /**
     * Establece la lista con los apoderados.
     * @param attorneyList	Lista con los apoderados.
     */
    public void setAttorneyList(List<User> attorneyList) {
        this.attorneyList = attorneyList;
    }

    /**
     * Devuelve el nombre por el que se realizará la búsqueda del apoderado.
     * @return	Nombre completo a buscar.
     */
    public String getAttorneyName() {
        return attorneyName;
    }

    /**
     * Establece el nombre por el que se realizará la búsqueda del apoderado.
     * @param attorneyName	Nombre completo a buscar.
     */
    public void setAttorneyName(String attorneyName) {
        this.attorneyName = attorneyName;
    }
}