package edu.uoc.tfc.j2ee.icampanario.grepresenta.telematic.actions;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.CitizensManager;
import edu.uoc.tfc.j2ee.icampanario.components.citizensManager.exception.CitizensManagerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentDataModel;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.State;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.EmpowermentQueryServiceDAO;

/**
 * Action para la vista de las bandejas dónde el usuario
 * es poderdante o apoderado
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("overviewAction")
@Scope(ScopeType.SESSION)
public class OverviewAction implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = -6109101608744899292L;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * Componente mensajes
	 */
	@In(required = false)
	private FacesMessages facesMessages;

	/**
	 * CitizensManager
	 */
	@In(required = false, scope = ScopeType.SESSION)
	private CitizensManager citizensManager;

	/**
	 * Usuario establecido
	 */
	@In(scope = ScopeType.SESSION, required = false)
	private User user;

	/**
	 * Dao de apoderamientos
	 */
	@In(create = true)
	private EmpowermentQueryServiceDAO empowermentService;

	/**
	 * Lista de apoderamientos como poderdante
	 */
	@DataModel
	private List<EmpowermentDataModel> empAsGrantor;

	/**
	 * Lista de apoderamientos como apoderado
	 */
	@DataModel
	private List<EmpowermentDataModel> empAsAttorney;

	/**
	 * Mapa con los estados de los apoderamientos
	 */
	private Map<Long,State> mapsEmpowermentState = new HashMap<Long, State>();

	//Filtros

	/**
	 * Filtro procedimiento en grantor list
	 */
	private String filterProcedureG;

	/**
	 * Filtro actual.
	 */
	private String filterValue = "";


	/**
	 * Obtiene los apoderados como poderdante
	 */
	@Factory("empAsGrantor")
	@Observer("refreshGrantorList")
	public void retrieveEmpowermentAsGrantor() {

		//Se reinician los estados y esta llamada sirve para las dos listas de apoderamientos
		refreshStates();

		empAsGrantor = new LinkedList<EmpowermentDataModel>();
		List<Empowerment> auxEmp = empowermentService.getEmpowermentsAsGrantor(user.getId());
		try {
			for (Empowerment emp : auxEmp) {
				empAsGrantor.add(new EmpowermentDataModel(emp, mapsEmpowermentState.get(emp.getId()), null, citizensManager.get(emp.getIdAttorney())));
			}
		} catch (CitizensManagerException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
					"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}

	}

	/**
	 * Obtiene los apoderados como apoderado
	 */
	@Factory("empAsAttorney")
	@Observer("refreshAttorneyList")
	public void retrieveEmpowermentAsAttorney() {
		empAsAttorney = new LinkedList<EmpowermentDataModel>();

		List<Empowerment> auxEmp = empowermentService.getEmpowermentAsAttorney(user.getId());
		try {
			for (Empowerment emp : auxEmp) {
				empAsAttorney.add(new EmpowermentDataModel(emp, mapsEmpowermentState.get(emp.getId()), citizensManager.get(emp.getIdGrantor()), null));
			}
		} catch (CitizensManagerException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
					"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Método que busca en la lista los que cumplan el criterio de búsqueda.
	 */
	public void filterLists() {
		List<EmpowermentDataModel> empAsGrantorAux = new ArrayList<EmpowermentDataModel>();
		List<EmpowermentDataModel> empAsAttorneyAux = new ArrayList<EmpowermentDataModel>();

		retrieveEmpowermentAsGrantor();
		retrieveEmpowermentAsAttorney();

		for (EmpowermentDataModel emp : empAsGrantor) {
			if (filter(emp)) {
				empAsGrantorAux.add(emp);
			}
		}

		for (EmpowermentDataModel emp : empAsAttorney) {
			if (filter(emp)) {
				empAsAttorneyAux.add(emp);
			}
		}
		empAsGrantor = empAsGrantorAux;
		empAsAttorney = empAsAttorneyAux;
	}

	/**
	 * Filtra los objetos para la búsqueda.
	 * @param current Objeto en el q buscar.
	 * @return 	true: Si se ha encontrado coincidencia en el objeto
	 * 			false: Si no se ha encontrado coincidencias.
	 */
	public boolean filter(Object current) {
		EmpowermentDataModel currentString = (EmpowermentDataModel)current;
		if (filterValue == null || filterValue.length() == 0) {
			return true;
		}
		boolean result = false;

		if (currentString.getEmpowermentNumber() != null
				&& currentString.getEmpowermentNumber().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getCurrentState() != null
				&& currentString.getCurrentState().getName() != null
				&& currentString.getCurrentState().getName().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getAttorney() != null
				&& currentString.getAttorney().getName() != null
				&& currentString.getAttorney().getName().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getAttorney() != null
				&& currentString.getAttorney().getSurname() != null
				&& currentString.getAttorney().getSurname().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getGrantor() != null
				&& currentString.getGrantor().getName() != null
				&& currentString.getGrantor().getName().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getGrantor() != null
				&& currentString.getGrantor().getSurname() != null
				&& currentString.getGrantor().getSurname().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getProcedure() != null
				&& currentString.getProcedure().getDescription() != null
				&& currentString.getProcedure().getDescription().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getValidityInitDateDesc() != null
				&& currentString.getValidityInitDateDesc().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}
		if (currentString.getValidityEndDateDesc() != null
				&& currentString.getValidityEndDateDesc().toUpperCase().contains(filterValue.toUpperCase())) {
			result = true;
		}

		return result;
	}

	/**
	 * Refresca todos los estados
	 */
	@Observer("refreshStates")
	public void refreshStates() {
		mapsEmpowermentState = empowermentService.getCurrentStateForAllEmpowerment();
	}


	/**
	 * Modifica la lista de apoderamientos como poderdante
	 * @param empAsGrantor nueva lista de apoderamientos como poderdante
	 */
	public void setEmpAsGrantor(List<EmpowermentDataModel> empAsGrantor) {
		this.empAsGrantor = empAsGrantor;
	}

	/**
	 * Devuelve la lista de apoderamientos como poderdante
	 * @return lista de apoderamientos como poderdante
	 */
	public List<EmpowermentDataModel> getEmpAsGrantor() {
		return empAsGrantor;
	}

	/**
	 * Modifica la lista de apoderamientos como apoderado
	 * @param empAsAttorney nueva lista de apoderamientos como apoderado
	 */
	public void setEmpAsAttorney(List<EmpowermentDataModel> empAsAttorney) {
		this.empAsAttorney = empAsAttorney;
	}

	/**
	 * Devuelve la lista de apoderamientos como apoderado
	 * @return la lista de apoderamientos como apoderado
	 */
	public List<EmpowermentDataModel> getEmpAsAttorney() {
		return empAsAttorney;
	}

	/**
	 * Modifica el mapa con los estados de apoderamientos
	 * @param mapsEmpowermentState nuevos estados de apoderamientos
	 */
	public void setMapsEmpowermentState(Map<Long,State> mapsEmpowermentState) {
		this.mapsEmpowermentState = mapsEmpowermentState;
	}

	/**
	 * Devuelve el mapa con los estados de apoderamiento
	 * @return mapa con los estados de apoderamiento
	 */
	public Map<Long,State> getMapsEmpowermentState() {
		return mapsEmpowermentState;
	}

	/**
	 * Modifica el filtro de procedimientos en grantor list
	 * @param filterProcedureG nuevo valor
	 */
	public void setFilterProcedureG(String filterProcedureG) {
		this.filterProcedureG = filterProcedureG;
	}

	/**
	 * Devuelve el filtro de procedimientos en grantor list
	 * @return filtro de procedimientos en grantor list
	 */
	public String getFilterProcedureG() {
		return filterProcedureG;
	}

	/**
	 * Devuelve el valor de filterValue
	 * @return filterValue
	 */
	public String getFilterValue() {
		return filterValue;
	}

	/**
	 * Establece el valor de filterValue
	 * @param filterValue filterValue
	 */
	public void setFilterValue(String filterValue) {
		this.filterValue = filterValue;
	}
}
