package edu.uoc.tfc.j2ee.icampanario.grepresenta.telematic.actions;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.io.StringWriter;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.Base64;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;

import edu.uoc.tfc.j2ee.icampanario.components.documentsManager.exception.DocumentsManagerException;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignTransaction;
import edu.uoc.tfc.j2ee.icampanario.components.signer.Signer;
import edu.uoc.tfc.j2ee.icampanario.components.signer.SignerFactory;
import edu.uoc.tfc.j2ee.icampanario.components.signer.exception.SignerException;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Document;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.Empowerment;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.EmpowermentState;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.State;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.services.EmpowermentQueryServiceDAO;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.PDFDocument;

/**
 * Action para rechazar un apoderamiento
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Name("rejectEmpAction")
@Scope(ScopeType.SESSION)
public class RejectEmpowermentAction implements Serializable {

	/**
	 * Atributo para serializar
	 */
	private static final long serialVersionUID = -4345504483272429383L;

	/**
	 * Configuracion de la aplicacion
	 */
	@In
	private Properties configurationgr;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * Componente mensajes
	 */
	@In(required = false)
	private FacesMessages facesMessages;

	/**
	 * Mensajes para internacionalización de la aplicación.
	 */
	@In
	private Map<String, String> messages;

	/**
	 * Eventos seam
	 */
	@In
	private Events events;

	/**
	 * Usuario establecido
	 */
	@In(scope = ScopeType.SESSION, value = "user", required = false)
	private User attorney;

	/**
	 * Dao de apoderamientos
	 */
	@In(create = true)
	private EmpowermentQueryServiceDAO empowermentService;

	/**
	 * Mapa de estados para no tener que consultar
	 */
	@In(required = false, scope = ScopeType.APPLICATION)
	private Map<String,State> states;

	/**
	 * Estado antiguo
	 */
	private EmpowermentState empoStateOld;


	/**
	 * Transaction
	 */
	private SignTransaction transaction;

	/**
	 * Atributo para firma
	 */
	private String clientSignerConfig ;

	/**
	 * Apoderamientos relacionados
	 */
	private List<Empowerment> relatedEmpowerments;

	/**
	 * Contiene el documento de rechazo.
	 */
	private Document rejectDoc;

	/**
	 * Prepara el documento de rechazo
	 * @param empowermentState estado del apoderamiento
	 * @param grantor apoderado
	 */
	@Observer("prepareRejectEmpowerment")
	public void prepareReject(EmpowermentState empowermentState, User grantor) {
		empoStateOld = empowermentState;

		EmpowermentDocument grantingDoc = new EmpowermentDocument();
		try {
			grantingDoc = empowermentService.getDocumentsForEmpowerment(empoStateOld.getEmpowerment().getId()).get(0);
		} catch (DocumentsManagerException e1) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
			"#{messages['message.error.administrator']}");
			log.error(e1.getMessage(), e1);
		}

		setRelatedEmpowerments(new LinkedList<Empowerment>());
		setRelatedEmpowerments(empowermentService.getRelatedEmpowerments(empoStateOld.getEmpowerment().getId(), grantingDoc.getIdDocument()));

		//Incluimos el actual
		relatedEmpowerments.add(empowermentState.getEmpowerment());

		FacesContext context = FacesContext.getCurrentInstance();
		String idiom = context.getExternalContext().getRequestLocale().getLanguage();

		String host = configurationgr.getProperty(GRepresentaConstants.OPENOFFICE_URL);
		int port = Integer.parseInt(configurationgr.getProperty(GRepresentaConstants.OPENOFFICE_PORT));

		InputStream openDocument = null;

		try {

			String separator = System.getProperty("file.separator");
			StringWriter urlRelative = new StringWriter();
			urlRelative.append("WEB-INF");
			urlRelative.append(separator);
			urlRelative.append("classes");
			urlRelative.append(separator);
			urlRelative.append("public");
			urlRelative.append(separator);
			urlRelative.append("forms_");
			urlRelative.append(idiom);
			urlRelative.append(separator);
			urlRelative.append("rechazo_telematica.odt");

			String urlReal = ((ServletContext)context.getExternalContext().getContext()).getRealPath(urlRelative.toString());

			if (urlReal != null) {
				openDocument = new FileInputStream(urlReal);

				Map<String,Object> vars = new HashMap<String,Object>();

				// variables comunes
				vars.put("poderdante", grantor.getName() + " " + grantor.getSurname());
				vars.put("nif_poderdante", grantor.getIdNumber());
				vars.put("apoderado", attorney.getName() + " " + attorney.getSurname());
				vars.put("nif_apoderado", attorney.getIdNumber());

				// procedimientos otorgamiento
				vars.put("tabla_procedimientos_rechazar", relatedEmpowerments);

				HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();

	            byte [] tmpFile = GRepresentaUseful.setVariables(host, port, openDocument, vars);
	            ByteArrayOutputStream out = new ByteArrayOutputStream(tmpFile.length);
	            GRepresentaUseful.generatePDF(host, port, out, tmpFile);

	            //Preparacion para la firma
	            Signer signer = SignerFactory.createInstance();

	            transaction = new SignTransaction();
	            SignDocument doc = new SignDocument();
	            doc.setContent(out.toByteArray());
	            doc.setMimeType("application/pdf");
	            doc.setName("rechazo_telematica.pdf");
	            transaction.addDocument(doc);
	            transaction.setUser(attorney.getCertificate());
	            signer.initSign(transaction, request);

	            clientSignerConfig = (String)request.getAttribute("clientSignerConfig");

			} else {
				facesMessages.add(Severity.ERROR, "#{messages['message.notFileFound']} " + urlRelative,
						"#{messages['message.notFileFound']} " + urlRelative);
			}
		} catch (FileNotFoundException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (Exception e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Descarga el documento de rechazo
	 */
	public void generateRejectedEmpowermentPdf() {
		FacesContext context = FacesContext.getCurrentInstance();

		try {

			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/pdf");
			response.setHeader("Content-disposition", "attachment;filename=\"rechazo_telematica.pdf\"");

            ServletOutputStream out = response.getOutputStream();

            out.write(transaction.getDocuments().get(0).getContent());

            out.flush();
            context.responseComplete();


		} catch (FileNotFoundException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (IOException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Firma el documento
	 * @return siguiente pantalla
	 */
	public String signDocument() {

		String next = null;

		HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();

		Signer signer = SignerFactory.createInstance();
		rejectDoc = null;

		try {

			signer.sign(transaction, request);

			updateEmpowermentStateAndSaveDocument();

			// procedimientos otorgamiento
			StringBuffer numExp = new StringBuffer();
			StringBuffer proc = new StringBuffer();
			for (Empowerment emp : relatedEmpowerments) {
				if (proc != null && proc.length() == 0) {
					proc.append(emp.getProcedure().getDescription());
				} else {
					proc.append(", ");
					proc.append(emp.getProcedure().getDescription());
				}
				if (numExp != null && numExp.length() == 0) {
					numExp.append(String.valueOf(emp.getId()));
				} else {
					numExp.append(", ");
					numExp.append(String.valueOf(emp.getId()));
				}
			}

			SignDocument signDocument = new SignDocument();
			signDocument.setContent(rejectDoc.getDocument());
			signDocument.setName("reject.pdf");
			signDocument.setMimeType("application/pdf");

			relatedEmpowerments = new LinkedList<Empowerment>();

			next = "showSignedRejectedDocumentEmpowerment";

		} catch (SignerException e) {
			log.error(e.getMessage(), e);
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
		}


		return next;
	}

	/***
	 * Actualiza el estado del apoderamiento
	 */
	public void updateEmpowermentStateAndSaveDocument() {

		boolean update = true;
		boolean createDoc = true;

		try {

			rejectDoc = new Document();
			rejectDoc.setDocument(transaction.getDocuments().get(0).getContent());
			rejectDoc.setFormat(transaction.getDocuments().get(0).getMimeType());
			rejectDoc.setPkcs7(Base64.decodeBase64(transaction.getDocuments().get(0).getSign()));
			rejectDoc.setSignDate(new GregorianCalendar());
			rejectDoc.setTransactionCode(transaction.getDocuments().get(0).getId());
			rejectDoc.setName("rechazo");
			rejectDoc.setType(GRepresentaConstants.DOC_TYPE_REQUEST);

			//Se crea el documento para luego relacionarlo
			empowermentService.createDocument(rejectDoc);

			for (Empowerment empowerment : relatedEmpowerments) {
				//Estado anterior
				EmpowermentState oldState = empowermentService.getStatesForEmpowerment(empowerment.getId()).get(0);

				EmpowermentState newState = new EmpowermentState();
				newState.setEmpowerment(empowerment);
				newState.setState(states.get(GRepresentaConstants.STATE_REJECTED));

				if (oldState != null) {
					update = empowermentService.updateState(oldState, newState) && update;
					createDoc = empowermentService.createEmpowermentDocument(empowerment, rejectDoc.getId(), null) && createDoc;
				} else {
					facesMessages.add(Severity.ERROR, "There is not current state.");
				}
			}

			if (!update) {
				facesMessages.add(Severity.ERROR, "Can´t change the state to 'Rechazado'");
			}
			if (!createDoc) {
				facesMessages.add(Severity.ERROR, "Error creating document reject");
			}

			//Se limpian los atributos
			empoStateOld = null;
			events.raiseEvent("refreshStates");
			events.raiseEvent("refreshAttorneyList");

		} catch (DocumentsManagerException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Genera el pdf firmado para mostrarlo
	 */
	public void generateRejectedEmpowermentSignedPdf() {

		FacesContext context = FacesContext.getCurrentInstance();

		try {

			HttpServletResponse response = (HttpServletResponse) context.getExternalContext().getResponse();
			response.setContentType("application/pdf");
			response.setHeader("Content-disposition", "attachment;filename=\"rechazo_telematica_firmado.pdf\"");

            ServletOutputStream out = response.getOutputStream();

            String key = GRepresentaConstants.KEY_SIGN;
            String verifyURL = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_URL);
            String endpoint = configurationgr.getProperty(GRepresentaConstants.SIGN_ENDPOINT_SERVER);
            String verifyText = configurationgr.getProperty(GRepresentaConstants.SIGN_VERIFY_TEXT);

            PDFDocument pdfPie = GRepresentaUseful.generatePdtWithFoot(transaction.getDocuments().get(0), key, verifyURL, endpoint, verifyText);

            out.write(pdfPie.getContent());

            out.flush();
            out.close();
            context.responseComplete();

		} catch (FileNotFoundException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (IOException e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
				"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		} catch (Exception e) {
			facesMessages.add(Severity.ERROR, "#{messages['message.error.administrator']}",
			"#{messages['message.error.administrator']}");
			log.error(e.getMessage(), e);
		}
	}

	/**
	 * Limpia los datos de sesión.
	 */
	public void clearData() {
		//Se limpian los atributos
		transaction = new SignTransaction();
		clientSignerConfig = null;
		rejectDoc = null;
	}

	/**
	 * Modifica el atributo para la firma
	 * @param clientSignerConfig nuevo atributo para la firma
	 */
	public void setClientSignerConfig(String clientSignerConfig) {
		this.clientSignerConfig = clientSignerConfig;
	}

	/**
	 * Devuelve el atributo para la firma
	 * @return atributo para la firma
	 */
	public String getClientSignerConfig() {
		return clientSignerConfig;
	}

	/**
	 * Modifica la lista de apoderamientos relacionados
	 * @param relatedEmpowerments nueva lista
	 */
	public void setRelatedEmpowerments(List<Empowerment> relatedEmpowerments) {
		this.relatedEmpowerments = relatedEmpowerments;
	}

	/**
	 * Devuelve la lista de apoderamientos relacionados
	 * @return apoderamientos relacionados
	 */
	public List<Empowerment> getRelatedEmpowerments() {
		return relatedEmpowerments;
	}
}