package edu.uoc.tfc.j2ee.icampanario.grepresenta.useful;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.State;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManager;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;
import org.jboss.seam.log.Log;


/**
 * Realiza la carga de la configuración de la aplicación
 * a partir del fichero grepresenta.properties.
 * 
 * @author Ismael Campanario Cabrera
 *
 */
@Startup
@Name("configurationAction")
@Scope(ScopeType.APPLICATION)
public class GRepresentaConfiguration implements Serializable {

	/**
	 * Serial version
	 */
	private static final long serialVersionUID = -1140242260693432369L;

	/**
	 * Log
	 */
	@Logger
	private Log log;

	/**
	 * EntityManager
	 */
    @In
    private EntityManager entityManager;

	/**
	 * Configurator configutation
	 */
	@Out(required = false, scope = ScopeType.APPLICATION)
	private Properties configurationgr = new Properties();

	/**
	 * Mapa de estados para no tener que consultar
	 */
	@Out(required = false, scope = ScopeType.APPLICATION)
	private Map<String,State> states;

	/**
	*  Initialization settings
	*/
	@SuppressWarnings("unchecked")
	@Create
	public void initConfiguration() {

		log.info("Charging configuration...");
		
		PropertiesConfiguration config = null;
		try {
			config = new PropertiesConfiguration("grepresenta.properties");
		} catch (ConfigurationException e) {
			log.error("Properties for GRepresenta not charged", e);
		}
		
		Iterator<String> keys = config.getKeys();
		while(keys.hasNext()) {
			String key = keys.next();
			configurationgr.put(key, config.getProperty(key));
		}

		//Cargamos el mapa de estados
		List<State> states = new LinkedList<State>();
		this.states = new HashMap<String,State>();
		states = entityManager.createQuery("from State").getResultList();
    	for (State state : states) {
    		this.states.put(state.getCode(), state);
		}

		if (configurationgr == null) {
			log.error("Properties for GRepresenta not charged");
		}
	}

	/**
	 * Devuelve las propiedades de la aplicaci�n
	 * @return el properties
	 */
	public Properties getConfigurationgr() {
		return configurationgr;
	}

	/**
	 * Mapa de estados
	 * @return mapa de estados
	 */
	public Map<String, State> getStates() {
		return states;
	}

	/**
	 * Mapa de estados
	 * @param states mapa de estados
	 */
	public void setStates(Map<String, State> states) {
		this.states = states;
	}

	/**
	 * Modifica las propiedades de la aplicaci�n
	 * @param configurationgr el nuevo properties
	 */
	public void setConfigurationgr(Properties configurationgr) {
		this.configurationgr = configurationgr;
	}

	/**
	 * get log
	 * @return the log
	 */
	public Log getLog() {
		return log;
	}

	/**
	 * set log
	 * @param log the log to set
	 */
	public void setLog(Log log) {
		this.log = log;
	}

	/**
	 * get entityManager
	 * @return the entityManager
	 */
	public EntityManager getEntityManager() {
		return entityManager;
	}

	/**
	 * set entityManager
	 * @param entityManager the entityManager to set
	 */
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

}
