package edu.uoc.tfc.j2ee.icampanario.grepresenta.useful;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.security.Security;
import java.security.cert.CertStore;
import java.security.cert.X509Certificate;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.cms.Attribute;
import org.bouncycastle.asn1.cms.AttributeTable;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import com.lowagie.text.BadElementException;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Image;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;

import edu.uoc.tfc.j2ee.icampanario.components.signer.SignDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.constants.GRepresentaNumericConstants;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.persistence.domain.User;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.certs.Certificate;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.certs.DNIECertificateImpl;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.certs.FNMTCertificateImpl;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.office.OfficeTools;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.PDFDocument;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.ReportSignaturePDFDecorator;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.SignerData;
import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report.SimplePDFDocument;

/**
 * Clase de utilidades
 * 
 * @author Ismael Campanario Cabrera
 *
 */
public class GRepresentaUseful {

	private static final Log LOG = LogFactory.getLog(GRepresentaUseful.class);
	
	/**
	 * Constructor privado clase de utilidad
	 */
	private GRepresentaUseful() { }

	/**
	 * Método para sustitucion de variables de un documento
	 * @param host maquina
	 * @param port puerto
	 * @param inputDocument documento entrada
	 * @param vars variables
	 * @return contenido
	 * @throws Exception si ha ocurrido un error
	 */
	public static byte[] setVariables(String host, int port, InputStream inputDocument, Map<String,Object> vars) throws Exception {

		byte[] document = new OfficeTools().getFreemarkerVars(host, port, inputDocument, vars);

		inputDocument.close();

		return document;
	}

	/**
	 * Método para generacion de pdf
	 * @param host maquina
	 * @param port puerto
	 * @param document documento fisico
	 * @param outputDocument documento salida
	 * @throws Exception si ha ocurrido un error
	 */
	public static void generatePDF(String host, int port, OutputStream outputDocument, byte[] document) throws Exception {

		final String ODT_TYPE = "odt";
		final String PDF_TYPE = "pdf";

		InputStream sourceDocument = new ByteArrayInputStream(document);

		OfficeTools.convertToPdf(host, port, sourceDocument, ODT_TYPE, outputDocument, PDF_TYPE);

		outputDocument.close();
		sourceDocument.close();

	}

	/**
	 * Genera el pdf con el pie de firma
	 * @param document el documento pdf
	 * @param key key
	 * @param verifyURL url verificacion
	 * @param endpoint url servicio
	 * @param verifyText texto de verificacion
	 * @param type Tipo del cajetin de firma.
	 * @return pdf con pie de firma
	 * @throws PDFDocumentException si ha ocurrido algun error al generar documento
	 */
	public static PDFDocument generatePdtWithFoot(SignDocument document, String key, String verifyURL,
				String endpoint, String verifyText) throws Exception {

		PDFDocument pdfPie = null;

		SignerData signerData = getCertificateData(Base64.decodeBase64(document.getSign()));

        // Preparación del signerData
        signerData.setPrefix("");
        signerData.setTransactionId(document.getId());

        PDFDocument pdfWithoutSign = new SimplePDFDocument("documento.pdf", document.getContent());
        List<SignerData> signers = new LinkedList<SignerData>();
        signers.add(signerData);

        pdfPie = footerGenerator(pdfWithoutSign, signers, verifyURL, endpoint, key.split(","), verifyText);

		return pdfPie;

	}

	/**
	 * Devuelve el porcentaje de similitud entre un usuario dado y la lista de usuarios.
	 * @param userList Lista de usuarios.
	 * @param userName Nombre y apellidos del usuario a buscar.
	 * @param user Usuario encontrado.
	 * @return Porcentaje de similitud del nombre y apellido.
	 */
	public static int getSimilarUser(List<User> userList, User user, String userName) {
		int maxNumCarComunes = 0;
		for (User userActual : userList) {
			String fullName = userActual.getName() + " " + userActual.getSurname();
			int numCarComunes = GRepresentaUseful.similitud(userName, fullName, true);
			if (numCarComunes > maxNumCarComunes) {
				user.setName(userActual.getName());
				user.setSurname1(userActual.getSurname1());
				user.setSurname2(userActual.getSurname2());
				user.setId(userActual.getId());
				maxNumCarComunes = numCarComunes;
			}
		}
		String fullName = user.getName() + " " + user.getSurname();
		return maxNumCarComunes * GRepresentaNumericConstants.TEXT_100 / fullName.length();
	}

	/**
	 * Devuelve un número dependiendo del número de carácteres que tengan iguales dos cadenas.
	 * @param cad1 Cadena origen.
	 * @param cad2 Cadena a comparar.
	 * @param ignoreCase Indica si se quiere comprobar las mayúsculas y minúsculas.
	 * @return Número de coincidencias.
	 */
	private static int similitud(String cad1, String cad2, boolean ignoreCase) {

		//int maxPasos = 1;//cad1.length() > cad2.length() ? cad1.length() : cad2.length() ;
	  	int maxPasos = cad1.length() > cad2.length() ? cad1.length() : cad2.length() ;

		String separador = "#";
		StringBuffer cadComun = new StringBuffer();

		int saltoEnc = 0;
		int numCarComunes = 0;
		int i1 = 0;
		int i2 = 0;
		int k1 = 0;
		int k2 = 0;
		int lon1 = cad1.length();
		int lon2 = cad2.length();

		if (ignoreCase) {
			cad1 = cad1.toLowerCase();
			cad2 = cad2.toLowerCase();
		}

		if (maxPasos <= 1 || cad1 == null || cad2 == null || lon1 == 0 || lon2 == 0) {
			return 0;
		}

		while (true) {
			if (i1 > lon1 || i2 > lon2) {
				break;
			}


			int n1 = lon1 <= i1 + 1 ? lon1 : (i1 + 1);
			int n2 = lon2 <= i2 + 1 ? lon2 : (i2 + 1);
			if (cad1.substring(i1,n1).equals(cad2.substring(i2,n2))) {
				cadComun.append(cad1.substring(i1,n1));
				numCarComunes++;
				i1++;
				i2++;
			} else {
				saltoEnc = 0;
				k1 = 0;
				k2 = 0;

				while (true) {
					while (true) {
						if (k1 == 0 && k2 == 0) {
							k2 = 1;
						}

						n1 = lon1 <= i1 + k1 + 2 ? lon1 : (i1 + k1 + 2);
						n2 = lon2 <= i2 + k2 + 2 ? lon2 : (i2 + k2 + 2);

						String s1 = cad1.substring(i1 + k1 > n1 ? n1 : (i1 + k1),n1);
						String s2 = cad2.substring(i2 + k2 > n2 ? n2 : (i2 + k2),n2);
						if (s1.equals(s2)) {
							if (lon1 > i1 + k1 + 2) {
								cadComun.append(separador);
								numCarComunes = numCarComunes + 2;
							}
							cadComun.append(cad1.substring(i1 + k1 > n1 ? n1 : (i1 + k1),n1));
							saltoEnc = 1;
							i1 = i1 + k1 + 2;
							i2 = i2 + k2 + 2;

							k1 = maxPasos + 1;
				            k2 = maxPasos + 1;
						}

						k2++;

						if (k2 > maxPasos) {
							k2 = 0;
							k1++;
							break;
						}
					}

					if (k1 > maxPasos) {
						break;
					}
				}

				if (saltoEnc == 0) {
					break;
				}
			}
		}

		return numCarComunes;
	}

	/**
	 * Devuelve la referencia a la marca de agua
	 * @param idiom idioma
	 * @return url
	 */
	public static String getWaterMarkReference(String idiom) {
		StringWriter urlRelative = new StringWriter();
		String separator = System.getProperty("file.separator");
		urlRelative.append("WEB-INF");
		urlRelative.append(separator);
		urlRelative.append("classes");
		urlRelative.append(separator);
		urlRelative.append("public");
		urlRelative.append(separator);
		urlRelative.append("forms_");
		urlRelative.append(idiom);
		urlRelative.append(separator);
		urlRelative.append("watermark.gif");
		return urlRelative.toString();
	}
	
	/**
	 * Obtiene los datos del certificado.
	 *
	 * @param pkcs7	Archivo PKCS7 con los datos de firma.
	 * @return DataSigner con nombre, dni y fecha de firma.
	 */
	public static SignerData getCertificateData(byte[] pkcs7) {
		String format = "CMS";
		List<SignerData> signerDataList = getSignersDataCertificate(pkcs7, format);
		return signerDataList.get(signerDataList.size() - 1);
	}
	
	/**
	 * Obtiene los datos del certificado.
	 *
	 * @param pkcs7	Archivo PKCS7 o xml con los datos de firma.
	 * @param format Formato de firma del fichero de firma.
	 * @return Lista de DataSigner con nombres, dnis y fechas de firma.
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static List<SignerData> getSignersDataCertificate(byte[] pkcs7, String format) {
		List<SignerData> dataSignerList = new ArrayList<SignerData>();
		Security.addProvider(new BouncyCastleProvider());
		try {
			
			if (format.equalsIgnoreCase("CMS")) {
				CMSSignedData signedData = new CMSSignedData(pkcs7);
				CertStore certs = signedData.getCertificatesAndCRLs("Collection", "BC");
				SignerInformationStore signers = signedData.getSignerInfos();
				
				Collection<Object> c = signers.getSigners();
				Iterator<Object> it = c.iterator();
				while (it.hasNext()) {
					SignerInformation signer = (SignerInformation) it.next();
					Collection certCollection = certs.getCertificates(signer.getSID());
					Iterator certIt = certCollection.iterator();
					X509Certificate cert = (X509Certificate) certIt.next();
					String subject = cert.getSubjectDN().getName();
					String issuer = cert.getIssuerDN().getName();
					SignerData dataSigner = getDataSigner(subject, issuer, format);
					
					AttributeTable signedAttributes = signer.getSignedAttributes();
				    if (signedAttributes != null) {
				    	Attribute attribute = signedAttributes.get(new DERObjectIdentifier("1.2.840.113549.1.9.5"));
				    	if (attribute != null && attribute.getAttrValues()!= null && attribute.getAttrValues().size() > 0) {
				    		String fecha = attribute.getAttrValues().getObjectAt(0).toString();
						    if (fecha.matches("\\d{12}[zZ]")) {
						    	DateFormat df = new SimpleDateFormat("yyMMddHHmmss");
						    	Calendar calendar = new GregorianCalendar();
						    	calendar.setTime(df.parse(fecha.substring(0, fecha.length() - 1)));
						    	calendar.add(Calendar.MILLISECOND, TimeZone.getDefault().getOffset(calendar.getTimeInMillis()));

						    	dataSigner.setDate(calendar.getTime());
						    }
						    else {
						    	throw new ParseException("Formato fecha no soportado", 0);
						    }
				    	}
				    }
				    dataSignerList.add(dataSigner);
				}
			} else {
				LOG.error("El formato de firma elegido no está soportado.");
			}
		} catch (Exception e) {
			LOG.error("Se ha producido un error al obtener los datos de la firma.", e);
		}
		return dataSignerList;
	}
	
	/**
	 * Método auxiliar que realiza una copia de datos de un InputStream a un OutputStream
	 * @param input Origen de los datos a copiar
	 * @param output Destino dónde se copiarán los datos
	 * @throws IOException
	 */
	public static void copyDataStream(InputStream input, OutputStream output) throws IOException {

		final int BUFFER = 2048;
		int count;
		byte data[] = new byte[BUFFER];
		while ((count = input.read(data, 0, BUFFER)) != -1) {
			output.write(data, 0, count);
		}
	}

	/**
	 * Devuelve el mensaje de error cuando no se puede obtener el valor de una variable
	 * @param varName Nombre de la variable
	 * @return Mensaje de error para la variable
	 */
	public static String getErrorVar(String varName) {
		return "##NO SE PUDO OBTENER EL VALOR DE LA VARIABLE '" + varName + "'##";
	}

	/**
	 * Convierte un array de bytes a hexadecimal.
	 * @param data
	 * @return
	 */
	public static String convertToHex(byte[] data) {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < data.length; i++) {
			int halfbyte = (data[i] >>> 4) & 0x0F;
			int two_halfs = 0;
			do {
				if ((0 <= halfbyte) && (halfbyte <= 9))
					buf.append((char) ('0' + halfbyte));
				else
					buf.append((char) ('a' + (halfbyte - 10)));
				halfbyte = data[i] & 0x0F;
			} while (two_halfs++ < 1);
		}
		return buf.toString();
	}

	/**
	 * Convierte la cadena de la clave 3DES a un array de bytes.
	 * @param key
	 * @return
	 */
	public static byte[] keyToByteArray(String key){
		return keyToByteArray(key.split(","));
	}

	/**
	 * Convierte el array de cadenas de la clave 3DES a un array de bytes.
	 * @param key
	 * @return
	 */
	public static byte[] keyToByteArray(String []key){
	    byte[] binaryKey = new byte[key.length];
	    for (int i = 0; i < binaryKey.length; i++) {
	    	binaryKey[i] = (byte) Integer.parseInt(key[i].substring(2), 16);
	    }
	    return binaryKey;
	}
	
	/**
	 * Método que lee desde un inputstream y lo pasa a memoria en un array
	 *
	 * @param in Entrada de datos
	 * @return array con los bytes leidos
	 * @throws Exception
	 */
	public static byte[] inputStreamToByte(InputStream in) throws Exception {

		byte[] datos = null;

		try {
			if (in == null) {
				return null;
			}

			if (in.markSupported()) {
				in.mark(in.available());
			}

			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			byte[] buffer = new byte[GRepresentaConstants.BUFFER_SIZE];
			int leidos;
			while ((leidos = in.read(buffer)) > 0) {
				bos.write(buffer, 0, leidos);
			}
			datos = bos.toByteArray();
			bos.close();

			if (in.markSupported()) {
				in.reset();
			}

			in.close();
		} catch (Exception ex) {
			LOG.error("InputStreamToByte: " + ex.getMessage(), ex);
		}
		return datos;
	}

	/**
	 * Obtiene la fecha de firma.
	 * @param pkcs7 Fichero PKCS7 con los datos de firma
	 * @return fecha de firma
	 */
	@SuppressWarnings("unchecked")
	public static Date getSignDate(byte[] pkcs7) {
		Security.addProvider(new BouncyCastleProvider());
		try {

			CMSSignedData signedData = new CMSSignedData(pkcs7);
			SignerInformationStore signers = signedData.getSignerInfos();

			Collection<Object> c = signers.getSigners();
			Iterator<Object> it = c.iterator();

			while (it.hasNext()) {
			    SignerInformation signer = (SignerInformation) it.next();

			    AttributeTable signedAttributes = signer.getSignedAttributes();
			    if (signedAttributes != null) {
			    	Attribute attribute = signedAttributes.get(new DERObjectIdentifier("1.2.840.113549.1.9.5"));
			    	if (attribute != null && attribute.getAttrValues()!= null && attribute.getAttrValues().size() > 0) {
			    		String fecha = attribute.getAttrValues().getObjectAt(0).toString();
					    if (fecha.matches("\\d{12}[zZ]")) {
					    	DateFormat df = new SimpleDateFormat("yyMMddHHmmss");
					    	Calendar calendar = new GregorianCalendar();
					    	calendar.setTime(df.parse(fecha.substring(0, fecha.length() - 1)));
					    	calendar.add(Calendar.MILLISECOND, TimeZone.getDefault().getOffset(calendar.getTimeInMillis()));

					    	return calendar.getTime();
					    }
					    else {
					    	throw new ParseException("Formato fecha no soportado", 0);
					    }
			    	}
			    }
			}
		}
		catch (Exception e) {
			LOG.error("Se ha producido un error al obtener la fecha del fichero de firma", e);
		}
		return null;
	}
	
	/**
     * Parsea un documento XML y lo introduce en un DOM.
     * 
     * @param uriFirma URI al fichero XML
     * @return Documento parseado
     */
    public static Document parseaDoc(InputStream fichero) {
		
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true) ;

        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException ex) {
            System.out.println("Error interno al parsear la firma");
            ex.printStackTrace();
            return null;
        }

        Document doc = null;
        try {
            doc = db.parse(fichero);
            return doc;
        } catch (SAXException ex) {
            doc = null;
        } catch (IOException ex) {
            System.out.println("Error interno al validar firma");
            ex.printStackTrace();
        } finally {
            dbf = null;
            db = null; 
        }
		
        return null;
    }
    
    /**
	 * Parsea los datos del firmante.
	 * @param subject Asunto del certificado, con los datos del firmante.
	 * @param format Formato del fichero de firma.
	 * @return dataSigner Objeto signerData con los datos del firmante.
	 * @throws Exception 
	 */
	public static SignerData getDataSigner(String subject, String issuer, String format) throws Exception {
		LOG.info("Subject: " + subject);
		LOG.info("Issuer: " + issuer);

		subject = subject.replace("\\,", "");
		issuer = issuer.replace("\\,", "");
		
		String[] subjectDN = subject.split(",");
		String[] issuerDN = issuer.split(",");
		
		Map<String, String> subjectDNMap = new HashMap<String, String>();
		Map<String, String> issuerDNMap = new HashMap<String, String>();
		
		for (String subjectItem : subjectDN) {
			String[] item = subjectItem.split("=");
			
			String key = item[0]; 
			String value = item[1];
			
			if (subjectDNMap.containsKey(key)) {
				subjectDNMap.put(key, subjectDNMap.get(key) + ", " + value);
			} else {
				subjectDNMap.put(key, value);
			}
			
		}
		
		for (String issuerItem : issuerDN) {
			String[] item = issuerItem.split("=");
			
			String key = item[0]; 
			String value = item[1];
			
			issuerDNMap.put(key, value);
		}
		
		Certificate certificate = null;
		if (issuer.toUpperCase().indexOf(GRepresentaConstants.CERTIFICATE_TYPE_FNMT) > -1) {
			certificate = new FNMTCertificateImpl();
		} else if (issuer.toUpperCase().indexOf(GRepresentaConstants.CERTIFICATE_TYPE_DNIE) > -1) {
			certificate = new DNIECertificateImpl();
		} else {
			LOG.error("La firma ha sido realizada con un certificado, no soportado.");
			throw new Exception("La firma ha sido realizada con un certificado, no soportado.");
		}
		
		return certificate.parse(subjectDNMap);
	}
	
	/**
	 * Agrega un pie de firma al documento.
	 * @param document
	 * @param signers
	 * @param verifirmaUrl
	 * @param endPointAfirmaServer
	 * @param clave3DES
	 * @param verifyText
	 * @return
	 * @throws PDFDocumentException
	 */
	public static PDFDocument footerGenerator(PDFDocument document, List<SignerData> signers, String verifirmaUrl, String endPointAfirmaServer, String[] clave3DES, String verifyText) throws Exception {
		PDFDocument footer = new ReportSignaturePDFDecorator(document, signers, verifirmaUrl, endPointAfirmaServer, clave3DES, verifyText);

		return footer;
	}
	
	/**
	 * Codifica la clave 3DES
	 * @param key
	 * @param s
	 * @return
	 * @throws Exception
	 */
	public static String encrypt(String key, String s) throws Exception {
		return encrypt(keyToByteArray(key), s);
	}

	/**
	 * Genera el Código Seguro de Verificación (CSV)
	 * @param key			Clave 3DES
	 * @param s				Código de transacción.
	 * @return				CSV
	 * @throws Exception
	 */
	public static String encrypt(String[] key, String s) throws Exception {
		return encrypt(keyToByteArray(key), s);
	}

	/**
	 * Codifica la clave 3DES
	 * @param key
	 * @param s
	 * @return
	 * @throws Exception
	 */
	public static String encrypt(byte[] key, String s) throws Exception{
		String encrypt = null;
		SecretKeyFactory kf = SecretKeyFactory.getInstance("DES");
	    SecretKey secretKey = kf.generateSecret(new DESKeySpec(key));
	    Cipher cd = Cipher.getInstance("DES");
	    cd.init(Cipher.ENCRYPT_MODE, secretKey);
	    encrypt = Base64.encodeBase64String(cd.doFinal(s.getBytes()));

		return encrypt;
	}
	
	/**
	 * Crea una marca de agua en un documento
	 * @param document
	 * @param markPath
	 * @throws Exception
	 */
	public static void insertWaterMark(byte[] document, String markPath, OutputStream out, Integer positionX, Integer positionY) {

		try {

			PdfReader reader = new PdfReader(document);
			PdfStamper stamp = new PdfStamper(reader, out);
			PdfContentByte under = stamp.getUnderContent(1);

			Image img = Image.getInstance(markPath);
		    img.setAbsolutePosition(positionX, positionY);

		    under.addImage(img);

		    stamp.close();

		} catch (IOException e) {
			LOG.error(e.getMessage(), e);
		} catch (BadElementException be) {
			LOG.error(be.getMessage(), be);
		} catch (DocumentException de) {
			LOG.error(de.getMessage(), de);
		}

	}
}