package edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.report;

import java.awt.Color;
import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.PageSize;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.Barcode128;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfImportedPage;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfWriter;

import edu.uoc.tfc.j2ee.icampanario.grepresenta.useful.GRepresentaUseful;

/**
 * Genera el documento PDF con los datos de la firma digital
 * 
 * @author Ismael Campanario Cabrera
 * 
 */
public class ReportSignaturePDFDecorator implements PDFDocument {

	private static Log log = LogFactory.getLog(ReportSignaturePDFDecorator.class);

	private static BaseFont fuente_base = null;
	private static Font fuente_normal = null;

	static {
		try {
			fuente_base = BaseFont.createFont(BaseFont.HELVETICA, BaseFont.CP1252, BaseFont.EMBEDDED);
			fuente_normal = new Font(fuente_base, 9f, Font.NORMAL, Color.BLACK);
		} catch (Exception e) {
			log.error("Error", e);
		}
	}

	private PDFDocument document = null;
	private String verifirmaUrl = null;
	private String afirmaServer = null;
	private List<SignerData> signers = new ArrayList<SignerData>();
	private String verifyText = null;
	private String signDate = null;
	private String transactionId = null;

	public ReportSignaturePDFDecorator(PDFDocument document,
			List<SignerData> signers, String verifirmaUrl,
			String endPointAfirmaServer, String[] key, String verifyText) {

		this.document = document;
		this.verifirmaUrl = verifirmaUrl;
		this.afirmaServer = endPointAfirmaServer;

		for (SignerData signer : signers) {
			try {
				String prefix = signer.getPrefix() != null ? signer.getPrefix(): "";
				String transaction = signer.getTransactionId() != null ? signer.getTransactionId() : "";

				signer.setTransactionId(GRepresentaUseful.encrypt(key, prefix + transaction));
			} catch (Exception e) {
				log.error("Error al cifrar el id de transaccion:", e);
				signer.setTransactionId("");
			}
		}
		if (verifyText == null) {
			log.error("El datos de componente TEXTO_VERIFICA es nulo");
		}

		Collections.sort(signers);
		this.signers = signers;

		SimpleDateFormat formato = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
		this.signDate = formato.format(signers.get(0).getDate());

		this.transactionId = signers.get(0).getTransactionId() != null ? signers.get(0).getTransactionId() : "";

		this.verifyText = verifyText.replaceAll("##url##", this.verifirmaUrl)
				.replaceAll("##csv##", this.transactionId);
	}

	public byte[] getContent() throws Exception {
		try {
			float fScale = 0.85f;

			// CREACION DEL DOCUMENTO
			Document pdf = new Document();
			pdf.setMargins(pdf.leftMargin(), pdf.leftMargin(), 30f, 30f); // izquierda
																			// derecha
																			// arriba
																			// abajo
			// Se crea el buffer temporal que contiene el pdf
			ByteArrayOutputStream buffer = new ByteArrayOutputStream();
			PdfWriter writer = PdfWriter.getInstance(pdf, buffer);
			pdf.addCreationDate();
			pdf.addProducer();
			pdf.open();

			// Se emplea para genera el codigo de barras e incrustar el pdf
			// original
			PdfContentByte cb = writer.getDirectContent();

			// Espaciador
			float spacerWidths[] = { 100 }; // percentage
			PdfPTable spacer = new PdfPTable(spacerWidths);
			spacer.setSpacingBefore(0);
			spacer.setSpacingAfter(0);
			spacer.setWidthPercentage(90); // percentage

			PdfPCell cell = new PdfPCell();
			cell.setHorizontalAlignment(Element.ALIGN_CENTER);
			cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
			cell.setFixedHeight(PageSize.A4.getHeight() - pdf.topMargin()- pdf.bottomMargin() - 120f);
			cell.setBorder(PdfPCell.NO_BORDER);
			cell.setBorderWidth(0);
			spacer.addCell(cell);

			// documento original
			PdfReader reader = new PdfReader(document.getContent());
			int n = reader.getNumberOfPages(); // numero de paginas del original

			for (int i = 1; i <= n; i++) {
				Rectangle rect = reader.getPageSizeWithRotation(i);

				float ratioX = (PageSize.A4.getWidth() / rect.getWidth()) * fScale;
				float ratioY = (PageSize.A4.getHeight() / rect.getHeight()) * fScale;
				float ratio = (ratioX < ratioY) ? ratioX : ratioY;
				PdfImportedPage page = writer.getImportedPage(reader, i);
				cb.addTemplate(page, ratio, 0, 0, ratio,
						(PageSize.A4.getWidth() * (1 - ratio) / 2f),
						(PageSize.A4.getHeight() * (1 - ratio)));
				cb.setRGBColorStroke(0xC0, 0xC0, 0xC0);
				cb.stroke();

				getSingTable(pdf, writer, spacer, n, i);
			}

			// fin documento.
			pdf.close();
			return buffer.toByteArray();

		} catch (Exception e) {
			throw e;
		}
	}

	/**
	 * @param pdf
	 * @param writer
	 * @param spacer
	 * @param n
	 * @param i
	 * @throws DocumentException
	 */
	private void getSingTable(Document pdf, PdfWriter writer,
			PdfPTable spacer, int n, int i) throws DocumentException {
		// Tabla con pie de firma
		float widths[] = { 15, 45, 15, 25 };
		PdfPTable table = new PdfPTable(widths);
		table.setWidthPercentage(100f);
		table.setKeepTogether(true);
		table.addCell(cell(verifyText, fuente_normal, PdfPCell.ALIGN_CENTER,
				PdfPCell.BOX, 4));
		Iterator<SignerData> itSigners = signers.iterator();
		int cont = 0;
		while (itSigners.hasNext() && cont < 8) {
			SignerData signer = itSigners.next();
			table.addCell(cell("FIRMADO POR:\n", fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
			table.addCell(cell(signer.getName(), fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 3));
		}
		table.addCell(cell("ID. FIRMA", fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell(transactionId, fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell("FECHA Y HORA", fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell(signDate, fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell("SERVIDOR", fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell(afirmaServer, fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell("PÁGINA", fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(cell(i + "/" + n, fuente_normal, PdfPCell.ALIGN_CENTER, PdfPCell.BOX, 1));
		table.addCell(barcode(transactionId, writer, 80f, 7f, PdfPCell.BOX));

		pdf.add(spacer);
		pdf.add(table);
		pdf.newPage();
	}

	public String getName() {
		return "INFORME-" + document.getName();
	}

	private PdfPCell cell(String s, Font font, int align, int border,
			int colspan) {
		PdfPCell cell = new PdfPCell(new Phrase((s == null) ? " " : s, font));
		cell.setBackgroundColor(Color.WHITE);
		cell.setBorder(border);
		cell.setBorderColor(Color.BLACK);
		cell.setHorizontalAlignment(align);
		cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
		cell.setColspan(colspan);

		return cell;
	}

	private PdfPCell barcode(String value, PdfWriter writer, float size,
			float fontSize, int border) {
		// Se emplea para genera el codigo de barras e incrustar el pdf original
		PdfContentByte cb = writer.getDirectContent();

		// GENERACION DEL CODIGO DE BARRAS
		Barcode128 code128 = new Barcode128();
		code128.setCode(value);
		code128.setFont(fuente_base);
		code128.setSize(fontSize);

		Image image = code128.createImageWithBarcode(cb, Color.BLACK,
				Color.BLACK);
		image.scaleToFit(500f, 24.001865f);

		PdfPCell cell = new PdfPCell(new Phrase(new Chunk(image, 0, 0)));
		cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
		cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
		cell.setBorder(border);
		cell.setColspan(4);

		return cell;
	}
}
